/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * %W% %E%
 */

package com.sun.mail.mbox;

import java.io.*;

public class DefaultMailbox extends Mailbox {
    private String home;

    public DefaultMailbox() {
	home = System.getProperty("user.home");
    }

    public MailFile getMailFile(String user, String folder) {
	return new DefaultMailFile(filename(user, folder));
    }

    public String filename(String user, String folder) {
	try {
	    char c = folder.charAt(0);
	    if (c == File.separatorChar) {
		return folder;
	    } else if (c == '~') {
		int i = folder.indexOf(File.separatorChar);
		String tail = "";
		if (i > 0) {
		    tail = folder.substring(i);
		    folder = folder.substring(0, i);
		}
		return home + tail;
	    } else {
		if (folder.equalsIgnoreCase("INBOX"))
		    folder = "INBOX";
		return home + File.separator + folder;
	    }
	} catch (StringIndexOutOfBoundsException e) {
	    return folder;
	}
    }
}

class DefaultMailFile extends File implements MailFile {
    protected transient RandomAccessFile file;

    DefaultMailFile(String name) {
	super(name);
    }

    public boolean lock(String mode) {
	try {
	    file = new RandomAccessFile(this, mode);
	    return true;
	} catch (FileNotFoundException fe) {
	    return false;
	} catch (IOException ie) {
	    file = null;
	    return false;
	}
    }

    public void unlock() { 
	if (file != null) {
	    try {
		file.close();
	    } catch (IOException e) {
		// ignore it
	    }
	    file = null;
	}
    }

    public void touchlock() {
    }

    public FileDescriptor getFD() {
	if (file == null)
	    return null;
	try {
	    return file.getFD();
	} catch (IOException e) {
	    return null;
	}
    }
}
