/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * %W% %E%
 */

package com.sun.mail.mbox;

import java.io.*;

/**
 * Convert the various newline conventions to the local platform's
 * newline convention.
 */
public class NewlineOutputStream extends FilterOutputStream {
    private int lastb = -1;
    private static final byte[] newline;

    static {
	String s = null;
	try {
	    s = System.getProperty("line.separator");
	} catch (SecurityException sex) {
	    // ignore, should never happen
	}
	if (s == null || s.length() <= 0)
	    s = "\n";
	newline = new byte[s.length()];
	s.getBytes(0, s.length(), newline, 0);
    }

    public NewlineOutputStream(OutputStream os) {
	super(os);
    }

    public void write(int b) throws IOException {
	if (b == '\r') {
	    out.write(newline);
	} else if (b == '\n') {
	    if (lastb != '\r')
		out.write(newline);
	} else {
	    out.write(b);
	}
	lastb = b;
    }

    public void write(byte b[]) throws IOException {
	write(b, 0, b.length);
    }

    public void write(byte b[], int off, int len) throws IOException {
	for (int i = 0 ; i < len ; i++) {
	    write(b[off + i]);
	}
    }
}
