/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * %W% %E%
 */

package com.sun.mail.mbox;

import java.io.File;
import java.io.FileDescriptor;
import java.util.StringTokenizer;

public class UNIXFile extends File {
    protected static final boolean loaded;

    private static final long serialVersionUID = -7972156315284146651L;

    public UNIXFile(String name) {
	super(name);
    }

    static {
	boolean lloaded = false;
	try {
	    System.loadLibrary("mbox");
	    lloaded = true;
	} catch (UnsatisfiedLinkError e) {
	    String classpath = System.getProperty("java.class.path");
	    String sep = System.getProperty("path.separator");
	    StringTokenizer st = new StringTokenizer(classpath, sep);
	    while (st.hasMoreTokens()) {
		String path = st.nextToken();
		int i = path.length() - 7;	// 7 == "classes".length()
		if (i > 0 && path.substring(i).equals("classes")) {
		    // XXX - pathname is Solaris/SPARC specific
		    String lib = path.substring(0, i) + "lib/sparc/libmbox.so";
		    try {
			System.load(lib);
			lloaded = true;
			break;
		    } catch (UnsatisfiedLinkError e2) {
			continue;
		    }
		} else if ((i = path.length() - 8) > 0 &&
					path.substring(i).equals("mail.jar")) {
		    // XXX - pathname is Solaris/SPARC specific
		    String lib = path.substring(0, i) + "lib/sparc/libmbox.so";
		    try {
			System.load(lib);
			lloaded = true;
			break;
		    } catch (UnsatisfiedLinkError e2) {
			continue;
		    }
		}
	    }
	}
	loaded = lloaded;
	if (loaded)
	    initIDs(FileDescriptor.class, FileDescriptor.in);
    }

    /**
     * Return the access time of the file.
     */
    public static long lastAccessed(File file) {
	return lastAccessed0(file.getPath());
    }

    public long lastAccessed() {
	return lastAccessed0(getPath());
    }

    private static native void initIDs(Class fdClass, FileDescriptor stdin);

    /**
     * Lock the file referred to by fd.  The string mode is "r"
     * for a read lock or "rw" for a write lock.  Don't block
     * if lock can't be acquired.
     */
    public static boolean lock(FileDescriptor fd, String mode) {
	return lock(fd, mode, false);
    }

    /**
     * Lock the file referred to by fd.  The string mode is "r"
     * for a read lock or "rw" for a write lock.  If block is set,
     * block waiting for the lock if necessary.
     */
    private static boolean lock(FileDescriptor fd, String mode, boolean block) {
	//return loaded && lock0(fd, mode);
	if (loaded) {
	    boolean ret;
	    //System.out.println("UNIXFile.lock(" + fd + ", " + mode + ")");
	    ret = lock0(fd, mode, block);
	    //System.out.println("UNIXFile.lock returns " + ret);
	    return ret;
	}
	return false;
    }

    private static native boolean lock0(FileDescriptor fd, String mode,
								boolean block);

    public static native long lastAccessed0(String name);
}
