/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

/*
 * @(#)SizeTerm.java	1.9 07/05/04
 */

package javax.mail.search;

import javax.mail.Message;

/**
 * This class implements comparisons for Message sizes.
 *
 * @author Bill Shannon
 * @author John Mani
 */
public final class SizeTerm extends IntegerComparisonTerm {

    private static final long serialVersionUID = -2556219451005103709L;

    /**
     * Constructor.
     *
     * @param comparison	the Comparison type
     * @param size		the size
     */
    public SizeTerm(int comparison, int size) {
	super(comparison, size);
    }

    /**
     * The match method.
     *
     * @param msg	the size comparator is applied to this Message's size
     * @return		true if the size is equal, otherwise false 
     */
    public boolean match(Message msg) {
	int size;

	try {
	    size = msg.getSize();
	} catch (Exception e) {
	    return false;
	}

	if (size == -1)
	    return false;

	return super.match(size);
    }

    /**
     * Equality comparison.
     */
    public boolean equals(Object obj) {
	if (!(obj instanceof SizeTerm))
	    return false;
	return super.equals(obj);
    }
}
