/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.attachments.db;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.sql.DataSource;

import org.eclipse.ebpm.attachments.IAttachmentStore;
import org.eclipse.ebpm.metadb.attachments.model.Attachment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DBAttachmentStore implements IAttachmentStore {

	public static final String NON_JTA_DATASOURCE = "javax.persistence.nonJtaDataSource";
	private static final String PERSISTENCE_UNIT = "org.eclipse.ebpm.metadb.attachments";
	
	private static Logger logger = LoggerFactory.getLogger(DBAttachmentStore.class);
	
	private DataSource ds = null;
	
	private Map<String, DataSource> properties = null;
	
	private EntityManagerFactory emf = null;
	private EntityManager em = null;

	public DBAttachmentStore (DataSource ds) {
		this.ds = ds;
		this.properties = new HashMap<String, DataSource>();
	    properties.put(NON_JTA_DATASOURCE, ds);
	    emf = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT, properties);
		em = emf.createEntityManager();
	}
	

	@Override
	public void delete(String id) {
		
		EntityTransaction tx = null;
		try{
			tx = em.getTransaction();
			tx.begin();
			//Si potrebbe fare prima una em.find e poi una em.remove con la query faccio prima.
			Query query = em.createQuery("delete from Attachment where id = :id");
			query.setParameter("id", Long.parseLong(id));
			query.executeUpdate();
			tx.commit();
		}catch (Throwable e) {
			tx.rollback();
			throw new RuntimeException("Attachment deletion failure", e);
		}

	}

	@Override
	public DataHandler load(String id) {
		
		try{
			//eseguo la select di un singolo record
			Attachment attachment = em.find(Attachment.class, id);
		
			byte[] result = attachment.getData();
			int blen = (int) result.length;

			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			baos.write(result, 0, blen);

			baos.flush();
			baos.close();

			ByteArrayDataSource bads = new ByteArrayDataSource(baos.toByteArray(), "application/octet-stream");

			DataHandler dha = new DataHandler(bads);
			return dha;
		}catch (Throwable e) {
			throw new RuntimeException("Attachment deletion failure", e);
		}

	}


	@Override
	public String store(String name, String correlationID, DataHandler value) {
		
		EntityTransaction tx = null;
		try{
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			InputStream is = value.getInputStream();

			byte buf[] = new byte[4096];
			int read;

			while( (read = is.read(buf)) > -1 ) {
				baos.write(buf, 0, read);
			}
			buf = null;
			
			Attachment attachment = new Attachment();
			attachment.setCorrelationid(name);
			attachment.setData(baos.toByteArray());
					
			tx = em.getTransaction();
			tx.begin();
			
			//Committo il nuovo attachment
			em.persist(attachment);
			
			tx.commit();
			
			return "" + attachment.getId();
		}catch (Throwable e) {
			tx.rollback();
			throw new RuntimeException("Attachment deletion failure", e);
		}
	
	}

	private String attachmentStoreId = null;
	
	@Override
	public String getAttachmentStoreId() {
		return attachmentStoreId;
	}

}
