/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.components.base64;

//import javax.xml.transform.Source;
//import javax.xml.transform.Transformer;
//import javax.xml.transform.TransformerFactory;
//import javax.xml.transform.stream.StreamSource;

import java.io.UnsupportedEncodingException;

import org.apache.commons.codec.binary.Base64;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Base64Component extends BaseSpagicService {
	
	protected Logger logger = LoggerFactory.getLogger(Base64Component.class);
	
    
    
    public String base64 = null;
    public String charset = null;
    public boolean forcedEncoding = false;
	
    //private static Base64 base64i;
	
	public void init(){
		try{
			this.base64 = propertyConfigurator.getString("base64");
			/* Charset 		Description
			 * US-ASCII 	Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode character set 
			 * ISO-8859-1   ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1 
			 * UTF-8 		Eight-bit UCS Transformation Format 
			 * UTF-16BE 	Sixteen-bit UCS Transformation Format, big-endian byte order 
			 * UTF-16LE 	Sixteen-bit UCS Transformation Format, little-endian byte order 
			 * UTF-16 		Sixteen-bit UCS Transformation Format, byte order identified by an optional byte-order mark 
			 */
			this.charset = "UTF-8"; 
			// propertyConfigurator.getString("charset")==null?"UTF-8":propertyConfigurator.getString("charset");
			this.forcedEncoding = false;
			
			//base64i = new Base64();
			
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate Base64Component Component", e);
		}
	}
    
   
	
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		try{			
			Document inDocument;
			Document outDocument;
			if(this.base64.equalsIgnoreCase("encoder")){				
				try{
					inDocument = DocumentHelper.parseText(in.getBodyText());
					outDocument = DocumentHelper.createDocument();
					Element base64row = (Element) DocumentHelper.createElement("base64");
					base64row.setText(base64EncoderWrapper(inDocument.getDocument().getRootElement().asXML()));
					outDocument.add(base64row);
					out.setBody(outDocument.getDocument().getRootElement().asXML());
					
				} catch (DocumentException exx){
					if (this.forcedEncoding){
						out.setBody("<base64>" + base64EncoderWrapper(in.getBodyText()) + "</base64>");
					} else
					throw new DocumentException("ENCODER: Illegal Xml in Input: " + exx);
				}
			} // end Encoder
			if(this.base64.equalsIgnoreCase("decoder")){
			 try{
				inDocument = DocumentHelper.parseText(in.getBodyText());
				
				Element base64row = (Element)inDocument.getRootElement();
				if (base64row!=null){
					outDocument = DocumentHelper.parseText(base64DecoderWrapper(base64row.getText()));
					out.setBody(outDocument.getDocument().getRootElement().asXML());
					} else {
						throw new DocumentException("DECODER: NO base64 TAG in Input.");
					}
				} catch (DocumentException exx){
					throw new DocumentException("DECODER: Illegal Xml in Input: " + exx);
				}
			}
			
			return true;
		}catch (Throwable e) {
			logger.error(e.getMessage(), e);
			e.fillInStackTrace();
			throw new Exception();
		}
	}

   
   
   

    
	public String base64EncoderWrapper(String string){
		/* GLC using 1.3
		 * 
		 * static byte[] encodeBase64Chunked(byte[] binaryData)
		 * Encodes binary data using the base64 algorithm and chunks the encoded output into 76 character blocks
		 */
		
		try {
		 return new String(Base64.encodeBase64Chunked(string.getBytes(this.charset)));
		 } catch (UnsupportedEncodingException e) {
	            throw new IllegalStateException("encodeBase64Chunked " + this.charset + ": " + e);
	        }
	}
    
	public String base64DecoderWrapper(String string){
		/* GLC using 1.3
		 * 
		 * static boolean isArrayByteBase64(byte[] arrayOctect) 
		 * 
		 * byte[] decode(byte[] pArray) --> decodeBase64
		 * Decodes a byte[] containing characters in the Base64 alphabet.
		 */
		if(Base64.isArrayByteBase64(string.getBytes())){
		   try {
			return new String(Base64.decodeBase64(string.getBytes()), this.charset);
			 } catch (UnsupportedEncodingException e) {
		            throw new IllegalStateException("base64DecoderWrapper " + this.charset + " INVALID: " + e);
		        }
		} else return null; 
		
	}
   
    
    

 

}

