/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.components.jdbc.config;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.codec.binary.Base64;
import org.eclipse.ebpm.xmlutil.EXMLUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

/**
 * Factory that initialize query parameter based on configuration and on
 * values retrived from input message.
 * 
 *
 */
public class JDBCParameterFactory {
	private static final Logger log = LoggerFactory.getLogger(JDBCParameterFactory.class);
	private static JDBCParameterFactory singleton;
	
	
	public static JDBCParameterFactory getInstance() {
		if (singleton == null) {
			singleton = new JDBCParameterFactory();
		}
		return singleton;
	}
	
	
	
	public Object initializeParameter(org.w3c.dom.Document doc, EXMLUtils xmlUtils,
		QueryParameterConfig config) throws Exception {
		
		
		Object xmlValue = retrieveMessageInEnvelope(doc,xmlUtils,config.getXpath());
		
		log.debug("Converting value: "+xmlValue+" to type: "+getClassFromConfig(config));
		Object value = ConvertUtils
			.convert((String)xmlValue, getClassFromConfig(config));
		
		if ( isBase64(retrieveParamType(config)) ) {
			return Base64.decodeBase64(value.toString().getBytes());
		}
		
		return value;
	}
	
    private Object retrieveMessageInEnvelope(org.w3c.dom.Document doc, EXMLUtils xmlUtils, String xpathString)
		throws Exception {

       
        if (xpathString.startsWith("node(")){
        	String nodeSelectionXPath = xpathString.substring(xpathString.indexOf("(")+1,xpathString.indexOf(")"));
        	Element n = xmlUtils.selectNode(doc, nodeSelectionXPath);
        	if (n == null){
        		log.warn("Node Selected by expression ["+nodeSelectionXPath+"] is NULL");
        		return "";
        	}
        	return xmlUtils.asXML(n,true);
        }else{
        	
        	
        	String nmEnvelope =  xmlUtils.valueOf(doc, xpathString);
        	if (nmEnvelope == null){
        		log.warn("No nodes were selected by expression ["+xpathString+"]");
        	}else if (nmEnvelope.trim().length() == 0){
        		log.warn("Empty String were selected by expression ["+xpathString+"]");
        	}
        	return nmEnvelope;
        }
        
	}

	private boolean isBase64(String paramType) {
		return QueryParameterConfig.BINARY_BASE64_TYPE
			.equals(paramType);
	}
	
	private Class getClassFromConfig(QueryParameterConfig config)
		throws ClassNotFoundException {
		
		String pType = retrieveParamType(config);
		if ( isBase64(pType) ) {
			
			return String.class;
		}
		return Class.forName(pType);
	}
	
	private String retrieveParamType(QueryParameterConfig config){
		
		String t = config.getParamType();
		
		if (t.indexOf("(") > 0 && t.indexOf(")") > 0)
			t = t.substring(t.indexOf("(")+1, t.indexOf(")"));
		return t;
	}
}
