/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.components.xsdvalidator;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.io.DOMWriter;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.util.resources.IResource;


public class ValidateComponent extends BaseSpagicService {

	
	public static final String FAULT_FLOW = "FAULT_FLOW";
	public static final String FAULT_WITH_EXCEPTION = "FAULT_WITH_EXCEPTION";
	private Schema schema;
	private String schemaLanguage = "http://www.w3.org/2001/XMLSchema";
	private Source schemaSource;
	private boolean includeStackTraceInError = false;
	private String faultHandlingMethod = null;
	private IResource schemaResource = null;
	
	
	public void init(){
		try{
			
			includeStackTraceInError = propertyConfigurator.getBoolean("includeStackTraceInError");
			this.schemaResource = propertyConfigurator.getResource("schema");
            SchemaFactory factory = SchemaFactory.newInstance(schemaLanguage);
            schemaSource = new StreamSource(schemaResource.openStream());
            schema = factory.newSchema(schemaSource);
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate Validate Component", e);
		}
	}	

	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
		Validator validator = schema.newValidator();

        MessageAwareErrorHandler errorHandler = new MessageAggregatingErrorHandler("ERROR", "urn:eng:spagic3", includeStackTraceInError);
        validator.setErrorHandler(errorHandler);
        DOMResult result = new DOMResult();
        

       
            // Only DOMSource and SAXSource are allowed for validating
            // See http://java.sun.com/j2se/1.5.0/docs/api/javax/xml/validation/Validator.html#validate(javax.xml.transform.Source,%20javax.xml.transform.Result)
            // As we expect a DOMResult as output, we must ensure that the input is a 
            // DOMSource
        	
        	Document dom4jDocument = DocumentHelper.parseText(in.getBodyText());
        
            DOMSource src = new DOMSource(transformtoDOM(dom4jDocument));
            validator.validate(src, result);
            if (errorHandler.hasErrors()) {
             
                if (!faultHandlingMethod.equalsIgnoreCase(FAULT_FLOW)){
                	throw new Exception("Failed to validate against schema: " + schema );
                }else{
                	out.setBody(errorHandler.getErrorMessageAsXML());
                	return true;
                }
            } else {
                out.setBody(in.getBody());
                return true;
            }

    }
		
	 public org.w3c.dom.Document transformtoDOM(Document doc) throws DocumentException {
		    DOMWriter writer = new DOMWriter();
		    return writer.write(doc);
	}

	
	
    
	

}
