/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.ftp.marshaler;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;

/**
 * A pluggable strategy for turning a file or URL input source into
 * a normalized message.
 *
 * 
 */
public interface FileMarshaler {

    public static final String FILE_NAME_PROPERTY = "org.spagic3.file.name";
    public static final String TEMP_FILE_NAME_PROPERTY = "org.spagic3.file.name.temp";
    public static final String FILE_PATH_PROPERTY = "org.spagic3.file.path";
    public static final String FILE_CONTENT = "org.spagic3.file.content";
    
    /**
     * Return a unique identifier for the marshaler.
     * @return The Marshaler id
     */
    String getMarshalerId();
	
    /**
     * Converts the file stream to a normalized message.
     *
     * @param exchange the message exchange
     * @param message  the message to populate
     * @param in       the input stream
     * @param path     the name of the file, URI or URL
     */
    void readMessage(Exchange exchange, Message message, InputStream in, String path) throws IOException;

    /**
     * Creates a output file name for the given exchange when reading an inbound
     * message.
     *
     * @param exchange the inbound message exchange
     * @param message the inbound message
     * @return the file name or null if a file name could not be found or calculated
     */
    List<String> getOutputNames(Exchange exchange, Message message);
    
    /**
     * Writes the inbound message to the destination stream of the given name
     *
     * @param exchange the inbound message exchange
     * @param message the inbound message
     * @param out the output stream to write to
     * @param path
     */ 
    void writeMessage(Exchange exchange, Message message, String outputName, OutputStream out, String path) throws IOException;
}
