/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.http.adapters;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.security.Constants;
import org.eclipse.ebpm.security.SecurityToken;
import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.InterceptorProvider.Phase;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.bindings.http.HttpConstants;
import org.eclipse.ebpm.soap.bindings.soap.SoapFault;
import org.eclipse.ebpm.soap.bindings.soap.SoapVersion;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class SOAPInputProtocolAdapter implements IHTTPInputProtocolAdapter {
	
	private Map<Phase, InterceptorChain> chains = new HashMap<Phase, InterceptorChain>();
	
	
	public String getAdapterId() {
		return "SOAP";
	}
	

	public SOAPInputProtocolAdapter(){
	}
	
	@Override
	public Exchange createExchange(HttpServletRequest request, String mep,
			String sender, String target, PropertyConfigurator pc, Object binding) throws Exception {
		
		Binding<?> soapBinding = (Binding<?>)binding;
		String method = request.getMethod();
        org.eclipse.ebpm.soap.api.Message soapMessage = soapBinding.createMessage();
        
        
      
        soapMessage.put(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE, request.getContentType());
        Map<String, String> headers = soapMessage.getTransportHeaders();
        for (Enumeration<?> e = request.getHeaderNames(); e.hasMoreElements();) {
            String name = (String) e.nextElement();
            String value = request.getHeader(name);
            headers.put(name, value);
        }
        headers.put(HttpConstants.REQUEST_URI, request.getRequestURL().toString());
        headers.put(HttpConstants.CONTENT_TYPE, request.getContentType());
        headers.put(HttpConstants.REQUEST_METHOD, method);
        
        /*
         * Something weird happen before the listening port. It seems that some clients
         * seems to do a GET when they do a POST... Therefore I set the inputstream if 
         * it is not null
         */
//        if (HttpConstants.METHOD_POST.equals(method) || HttpConstants.METHOD_PUT.equals(method)) {
//            soapMessage.setContent(InputStream.class, request.getInputStream());
//        }
        InputStream temp = request.getInputStream();
        if (temp != null)
        	 soapMessage.setContent(InputStream.class, request.getInputStream());
        request.setAttribute(org.eclipse.ebpm.soap.api.Message.class.getName(), soapMessage);
        soapMessage.put(SpagicConstants.SPAGIC_SENDER, sender);
        if (target != null)
        	soapMessage.put(SpagicConstants.SPAGIC_TARGET, target);
        
        InterceptorChain phase = getChain(Phase.ServerIn, soapBinding);
        phase.doIntercept(soapMessage);
        Exchange exchange = (Exchange)soapMessage.getContent(Exchange.class);
        
        Map<QName, DocumentFragment> soapHeaders = soapMessage.getSoapHeaders();
        
        for (QName soapHeaderQName : soapHeaders.keySet()){
        	System.out.println(" QName "+ soapHeaderQName);
        	if (soapHeaderQName.getLocalPart().equalsIgnoreCase("spagic-target-service")){
        		DocumentFragment aSoapHeader = soapHeaders.get(soapHeaderQName);
        		String targetServiceId = aSoapHeader.getTextContent();
        		exchange.setProperty(SpagicConstants.SPAGIC_TARGET, targetServiceId);
        		
        		// Consider only the content of the payload, without the root element
        		Message inMessage = exchange.getIn();

				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = factory.newDocumentBuilder();
				InputSource source = new InputSource(new StringReader(inMessage.getBodyText()));
		        Document document = builder.parse(source);
		        
		        
				// The payload should have the following structure:
				// <payload>
				//		<message>
				//			message.getBody
				//		</message>
				//		<properties>
				//			message.getProperties
				//		</properties>
				//		<security>
				//			message.getSecurityToken
				//		</security>

		        Element payloadElement = document.getDocumentElement();
				
				NodeList nl = payloadElement.getChildNodes();
				for (int i = 0; i < nl.getLength(); i++) {
					Node node = nl.item(i);
					if (node.getNodeName().equals(Constants.SPAGIC_NS_ALIAS + Constants.MESSAGE_ELEMENT)) {
						
						Node messageElement = node;
				        ByteArrayOutputStream baos = new ByteArrayOutputStream();
				        nodeToStream(messageElement.getFirstChild(), baos);
				        inMessage.setBodyText(baos.toString());	
				        
					} else if (node.getNodeName().equals(Constants.SPAGIC_NS_ALIAS + Constants.PROPERTIES_ELEMENT)) {
						
						Node propertiesElement = (Node) node;
						NamedNodeMap nnm = propertiesElement.getAttributes();
						for (int j = 0; j < nnm.getLength(); j++) {
							Node nodeProp = nnm.item(j);
							inMessage.setHeader(nodeProp.getNodeName(), nodeProp.getNodeValue());
						}					
						
					} if (node.getNodeName().equals(Constants.SPAGIC_NS_ALIAS + Constants.SECURITY_ELEMENT)) {
						
						Element securityElement = (Element) node;
						SecurityToken st = new SecurityToken();
												
						// Add properties as attributes
						st.setCommonName(securityElement.getAttribute(Constants.COMMON_NAME));
						st.setOrganizationalUnit(securityElement.getAttribute(Constants.ORGANIZATIONAL_UNIT));
						st.setOrganization(securityElement.getAttribute(Constants.ORGANIZATION));
						st.setLocality(securityElement.getAttribute(Constants.LOCALITY));
						st.setState(securityElement.getAttribute(Constants.STATE));
						st.setCountry(securityElement.getAttribute(Constants.COUNTRY));
						
						// The properties element, if present, is the only child
						Element propElement = (Element) securityElement.getFirstChild();
												
						if (propElement != null) {
							NamedNodeMap nnm = propElement.getAttributes();
							for (int nodeInd = 0; nodeInd < nnm.getLength(); nodeInd++) {
								Node nodeProp = nnm.item(nodeInd);

								st.addProperty(nodeProp.getNodeName(), nodeProp.getNodeValue());
							}			
						}
						
						// Put the security token within the properties
						inMessage.setHeader(Constants.SECURITY_TOKEN, st);						
					}
				}

        	}
        	
        }
        
        return exchange;
	}
	
	public static void nodeToStream(Node element, OutputStream out) {
	    try {
	      DOMSource source = new DOMSource(element);
	      StreamResult result = new StreamResult(out);
	      TransformerFactory transFactory = TransformerFactory.newInstance();
	      Transformer transformer = transFactory.newTransformer();
	      transformer.transform(source, result);
	    } catch (Exception ex) {
	    }
	  }
	

	@Override
	public void sendAccepted(Exchange exchange, HttpServletRequest request,
			HttpServletResponse response, PropertyConfigurator pc, Object binding) throws Exception {
		
		 response.setStatus(HttpServletResponse.SC_ACCEPTED);
		
		
	}

	@Override
	public void sendError(Exchange exchange, Exception error,
			HttpServletRequest request, HttpServletResponse response, PropertyConfigurator pc, Object binding)
			throws Exception {
		Binding<?> soapBinding = (Binding<?>)binding;
		response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		org.eclipse.ebpm.soap.api.Message soapMessageIn = ( org.eclipse.ebpm.soap.api.Message ) request.getAttribute(org.eclipse.ebpm.soap.api.Message.class.getName());
		org.eclipse.ebpm.soap.api.Message soapMessageOut = soapBinding.createMessage(soapMessageIn);
		soapMessageOut.setContent(OutputStream.class, response.getOutputStream());
		soapMessageOut.setContent(Exchange.class, exchange);
		soapMessageOut.put(SoapVersion.class, soapMessageIn.get(SoapVersion.class));

        InterceptorChain phase = getChain(Phase.ServerOutFault, soapBinding);
        SoapFault soapFault;
        if (error instanceof SoapFault) {
            soapFault = (SoapFault) error;
        } else {
            soapFault = new SoapFault(error);
        }
        soapMessageOut.setContent(Exception.class, soapFault);
        phase.doIntercept(soapMessageOut);
		
	}

	@Override
	public void sendFault(Exchange exchange, Message fault,
			HttpServletRequest request, HttpServletResponse response,PropertyConfigurator pc, Object binding)
			throws Exception {
		
		Binding<?> soapBinding = (Binding<?>)binding;
		response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		org.eclipse.ebpm.soap.api.Message soapMessageIn = ( org.eclipse.ebpm.soap.api.Message ) request.getAttribute(org.eclipse.ebpm.soap.api.Message.class.getName());
		org.eclipse.ebpm.soap.api.Message soapMessageOut = soapBinding.createMessage(soapMessageIn);
		
		soapMessageOut.setContent(OutputStream.class, response.getOutputStream());
		soapMessageOut.setContent(Exchange.class, exchange);
		soapMessageOut.setContent(Message.class, fault);
		soapMessageOut.put(SoapVersion.class, soapMessageIn.get(SoapVersion.class));
       
        InterceptorChain phase = getChain(Phase.ServerOutFault, soapBinding);
        QName code = (QName) fault.getHeader(SpagicConstants.SOAP_FAULT_CODE);
        String reason = (String)fault.getHeader(SpagicConstants.SOAP_FAULT_REASON);
        SoapFault soapFault = new SoapFault(code, reason, null, null, fault.getBody());
        soapMessageOut.setContent(Exception.class, soapFault);
        phase.doIntercept(soapMessageOut);
		
	}

	@Override
	public void sendOut(Exchange exchange, Message out,
			HttpServletRequest request, HttpServletResponse response,PropertyConfigurator pc, Object binding)
			throws Exception {
		
		 Binding<?> soapBinding = (Binding<?>)binding;
		 response.setStatus(HttpServletResponse.SC_OK);
		 org.eclipse.ebpm.soap.api.Message soapMessageIn = ( org.eclipse.ebpm.soap.api.Message )  request.getAttribute(org.eclipse.ebpm.soap.api.Message.class.getName());
		 org.eclipse.ebpm.soap.api.Message soapMessageOut = soapBinding.createMessage(soapMessageIn);
		 soapMessageOut.put(HttpServletResponse.class,response);
		 soapMessageOut.setContent(OutputStream.class, response.getOutputStream());
		 soapMessageOut.setContent(Exchange.class, exchange);
		 soapMessageOut.setContent(Message.class, out);
		 SoapVersion version = soapMessageIn.get(SoapVersion.class); 
		 soapMessageOut.put(SoapVersion.class, version); 
		 response.setContentType(version.getSoapMimeType());
		 InterceptorChain phase = getChain(Phase.ServerOut, soapBinding);
	     phase.doIntercept(soapMessageOut);
	}
	
	
	
	 protected InterceptorChain getChain(Phase phase, Binding<?> binding) {
	        InterceptorChain chain = chains.get(phase);
	        if (chain == null) {
	            chain = binding.getInterceptorChain(phase);
	            chains.put(phase, chain);
	        }
	        return chain;
	    }

		@Override
		public void setProperty(Map<String, String> map) {
			// TODO Auto-generated method stub
			
		}
	
}
