/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.http.adapters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.InterceptorProvider.Phase;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.bindings.soap.SoapConstants;
import org.eclipse.ebpm.soap.interceptors.xml.StaxInInterceptor;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.mortbay.io.ByteArrayBuffer;
import org.mortbay.jetty.HttpMethods;

public class SOAPProtocolOutputAdapter implements IHTTPOutputProtocolAdapter {
	
	
	private Map<Phase, InterceptorChain> chains = new HashMap<Phase, InterceptorChain>();
	public String getAdapterId() {
		return "SOAP";
	}
	
	 protected String getLocationUri(Exchange exchange, org.eclipse.ebpm.messaging.api.Message inMsg, PropertyConfigurator pc)  {
	       if (inMsg.getHeader("locationURI") != null){
	    	   return (String)inMsg.getHeader("locationURI");
	       }else{
	    	   return pc.getString("locationURI");
	       }
	       
	}

	@Override
	public void fillJettyExchange(Exchange nmrExchange,
			SpagicJettyHTTPExchange jettyExchange, PropertyConfigurator pc, Object binding) {
		  	
		 	Binding<?> soapBinding = (Binding<?>)binding;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
	        Message soapMessage = soapBinding.createMessage();
	      
	        boolean mustSetOperationHeader = pc.getBoolean("mustSetOperation", false);
	        
	        String operationName = null;
	        if (mustSetOperationHeader){
	        	operationName = (String) pc.getString("operation");
	        	nmrExchange.setProperty("mustSetOperation", mustSetOperationHeader);
	        	nmrExchange.setProperty(SoapConstants.SOAP_ACTION_HEADER, operationName);
	        }
	       
	        soapMessage.setContent(Exchange.class, nmrExchange);
	        soapMessage.setContent(org.eclipse.ebpm.messaging.api.Message.class, nmrExchange.getIn(false));
	        soapMessage.setContent(OutputStream.class, baos);
	        nmrExchange.setProperty(Message.class.getName(), soapMessage);

	        InterceptorChain phaseOut = getChain(Phase.ClientOut, soapBinding);
	        phaseOut.doIntercept(soapMessage);
	        jettyExchange.setMethod(HttpMethods.POST);
	        org.eclipse.ebpm.messaging.api.Message inMsg = nmrExchange.getIn(false);
	        
	        jettyExchange.setURL(getLocationUri(nmrExchange, inMsg,pc));
	        jettyExchange.setRequestContent(new ByteArrayBuffer(baos.toByteArray()));
	        for (Map.Entry<String,String> entry : soapMessage.getTransportHeaders().entrySet()) {
	        	jettyExchange.addRequestHeader(entry.getKey(), entry.getValue());
	        }
		
	}

	@Override
	public void handleException(Exchange nmrExchange,
			SpagicJettyHTTPExchange jettyExchange, Throwable t, PropertyConfigurator pc, Object binding) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void handleResponse(Exchange nmrExchange,
			SpagicJettyHTTPExchange jettyExchange, PropertyConfigurator pc, Object binding) {
			try{
				Binding<?> soapBinding = (Binding<?>)binding;
				Message soapMessageRequest = (Message) nmrExchange.getProperty(Message.class.getName());
				nmrExchange.setProperty(Message.class.getName(), null);
				Message soapMessageResponse = soapBinding.createMessage(soapMessageRequest);
	       
				soapMessageResponse.setContent(Exchange.class, nmrExchange);
				soapMessageResponse.setContent(InputStream.class, new ByteArrayInputStream(jettyExchange.getResponse()));
				soapMessageResponse.put(StaxInInterceptor.ENCODING, "utf-8");
				InterceptorChain phaseOut = getChain(Phase.ClientIn,soapBinding);
				phaseOut.doIntercept(soapMessageResponse);
			}catch (Exception e) {
				throw new RuntimeException(e);
			}
		
	}
	
 
	 protected InterceptorChain getChain(Phase phase, Binding<?> binding) {
	        InterceptorChain chain = chains.get(phase);
	        if (chain == null) {
	            chain = binding.getInterceptorChain(phase);
	            chains.put(phase, chain);
	        }
	        return chain;
	    }

		@Override
		public void setProperty(Map<String, String> map) {
			// TODO Auto-generated method stub
			
		}
	 
}
