/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.quartz.adapters;

import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.ebpm.connectors.quartz.support.SpagicQuartzJob;
import org.eclipse.ebpm.messaging.api.Message;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

/**
 * The default implementation of the Quartz marshaler
 *
 * 
 */
public class DefaultQuartzAdapter implements IQuartzAdapter {

	private static final String dateFormat = "dd/MM/yyyy HH:mm:ss.SSS";

    /**
     * Converts the value to a String
     *
     * @param value the value to be coerced into a string
     * @return the value as a string or null if it cannot be converted
     */
    protected String asString(Object value) {
        return value != null ? value.toString() : null;
    }


    public void populateNormalizedMessage(Message message, JobExecutionContext context)
        throws JobExecutionException {
    	
    	JobDetail detail = context.getJobDetail();
        JobDataMap dataMap = detail.getJobDataMap();
        for (Iterator iter = dataMap.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            String key = (String) entry.getKey();
            if (!key.equals(SpagicQuartzJob.COMPONENT_NAME) && !key.equals(SpagicQuartzJob.ENDPOINT_NAME)) {
                Object value = entry.getValue();
                //message.setProperty(key, value);
                message.setHeader(key, value);
            }
        }
        
        SimpleDateFormat format =  new SimpleDateFormat(dateFormat);

        
    	// create document
		Document document = DocumentHelper.createDocument();
		// set UTF-8 encoding
		//document.setXMLEncoding("UTF-8");
		Element root = document.addElement("timer");
		Element name = root.addElement("name");
		name.setText(detail.getName());
		Element group = root.addElement("group");
		group.setText(detail.getGroup());
		Element fullname = root.addElement("fullname");
		fullname.setText(detail.getFullName());
		Element description = root.addElement("description");
		description.setText(detail.getDescription());
		Element fireTimeFormat = root.addElement("fireTime_format");
		fireTimeFormat.setText(dateFormat);
		Element fireTime = root.addElement("fireTime");
		fireTime.setText(format.format(context.getFireTime()));
		
		message.setBody(document.asXML());
    }

}
