/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.datasource;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbcp.BasicDataSourceFactory;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public class DataSource implements javax.sql.DataSource{
	
	private Logger logger = LoggerFactory.getLogger(DataSource.class);
	private javax.sql.DataSource internalDS = null;
	private BasicDataSourceFactory bdf = new BasicDataSourceFactory();
	
	protected void activate(ComponentContext componentContext){
		try{
			org.eclipse.ebpm.util.properties.PropertyConfigurator propertyConfigurator = new PropertyConfigurator(componentContext.getProperties());	
			this.internalDS = bdf.createDataSource(propertyConfigurator.asProperties());
		}catch (Exception e) {
			throw new IllegalStateException("Cannot istantiate DataSource", e);
		}
	}
	
	
	
	@Override
	public Connection getConnection() throws SQLException {
		try{
			return internalDS.getConnection();
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new SQLException(e);
		}
	}

	@Override
	public Connection getConnection(String username, String password)
			throws SQLException {
		
		return internalDS.getConnection(username, password);
	}

	@Override
	public PrintWriter getLogWriter() throws SQLException {
		
		return internalDS.getLogWriter();
	}

	@Override
	public int getLoginTimeout() throws SQLException {
		return internalDS.getLoginTimeout();
	}

	@Override
	public void setLogWriter(PrintWriter logWriter) throws SQLException {
		internalDS.setLogWriter(logWriter);
		
	}

	@Override
	public void setLoginTimeout(int loginTimeout) throws SQLException {
		internalDS.setLoginTimeout(loginTimeout);
		
	}

	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		
		throw new SQLException("Not Implemented");
	}
	
	
	public String getDriverClassName(){
		return ((BasicDataSource)internalDS).getDriverClassName();
	}
	public String getUsername() {
		return ((BasicDataSource)internalDS).getUsername();
	}
	
	public String getUrl() {
		return ((BasicDataSource)internalDS).getUrl();
	}
	
	public String getPassword() {
		return ((BasicDataSource)internalDS).getPassword();
	}
	
	

	
	
	
}
