/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.dirwatcher;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.Element;
import org.eclipse.ebpm.constants.SpagicConstants;

public class GenericDeploymentDescriptor {

	private Hashtable<String, String> properties = new Hashtable<String, String>();
	private Hashtable<String, String> attributes = new Hashtable<String, String>();
	
	public Hashtable<String, String> getProperties() {
		return properties;
	}


	public GenericDeploymentDescriptor(Document document){
		Element rootElement = document.getRootElement();
		
		List<Attribute> rootElAttributes = rootElement.attributes();
		
		String localAttributeName = null;
		for ( Attribute anAttribute : rootElAttributes){
			localAttributeName = anAttribute.getName();
			this.attributes.put(localAttributeName, anAttribute.getValue());
			
			if (localAttributeName.equals("id") || 
			   localAttributeName.equalsIgnoreCase(SpagicConstants.SPAGIC_ID_PROPERTY) ||
			   localAttributeName.equalsIgnoreCase("factory.name"))
			   
			   this.properties.put(localAttributeName, anAttribute.getValue());
		}
		
		List<Element> propertiesNode = rootElement.elements("property");
		List<Element>  xpropertiesNode = rootElement.elements("xproperty");
		
		for (Element pn : propertiesNode){
			properties.put(pn.attributeValue("name"), pn.attributeValue("value"));
		}
		
		for (Element pn : xpropertiesNode){
			Iterator<Element> elIterator = ((Element)pn).elementIterator();
			if (elIterator.hasNext()){
				String value = elIterator.next().asXML();
				properties.put(pn.attributeValue("name"), value);
			}else{
				throw new IllegalStateException("XProperties error - an XProperty Must Have a Child Element");
			}
		}
	}
	
	public String getFactoryName() {
		return attributes.get("factory.name");
	}


	public String getId() {
		return attributes.get("id");
	}
	
	public String getSpagicId(){
		return attributes.get(SpagicConstants.SPAGIC_ID_PROPERTY);
	}
}
