/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.management.resources;

import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.datasource.IDataSourceManager;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;


public class GetResourceService extends BaseSpagicService {
	
	
	protected final AtomicReference<IDataSourceManager> datasourceManager = new AtomicReference<IDataSourceManager>(null); 
	
	

	public IDataSourceManager getDataSourceManager() {
		return this.datasourceManager.get();
	}
	
	public void setDataSourceManager(IDataSourceManager dsm) {
		this.datasourceManager.set(dsm);
	}
	
	public void unsetDataSourceManager(IDataSourceManager dsManager) {
		this.datasourceManager.compareAndSet(dsManager, null);
	}
	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
		Document doc =  DocumentHelper.createDocument();
		Element rootElement = doc.addElement("DATASOURCES");
		Set<String> ids = getDataSourceManager().getRegisteredDatasourcesIDs();
		DataSource ds = null;
		for (String id : ids){
			ds = getDataSourceManager().getDataSource(id);
			Element dsElement = rootElement.addElement("DATASOURCE");
			dsElement.addAttribute("name", id);
			if (ds instanceof org.eclipse.ebpm.datasource.DataSource){
				dsElement.addAttribute("driver",((org.eclipse.ebpm.datasource.DataSource)ds).getDriverClassName());
				dsElement.addAttribute("user",((org.eclipse.ebpm.datasource.DataSource)ds).getUsername());
				dsElement.addAttribute("url",((org.eclipse.ebpm.datasource.DataSource)ds).getUrl());
				dsElement.addAttribute("password",((org.eclipse.ebpm.datasource.DataSource)ds).getPassword());
			}
		}
		out.setBody(doc.asXML());
		return true;
	}

}
