/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.management.resources;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.lang.StringEscapeUtils;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.core.ISpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.osgi.framework.ServiceReference;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class ServicesConnectorsStatusService extends BaseSpagicService {


	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		String s = (String)in.getBodyText();
		if (s == null) {
			out.setBody("");
			return true;
		}
		
		Map<String, ServiceReference> serviceMap = new HashMap<String, ServiceReference>();
		Map<String, ServiceReference> connectorMap = new HashMap<String, ServiceReference>();
		

		InputSource inputSource = new InputSource();
		inputSource.setCharacterStream(new StringReader(s));
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder db = factory.newDocumentBuilder();
	 
	    Document payload = db.parse(inputSource);
	    
	    inputSource.setCharacterStream(
	    		new StringReader(
	    				StringEscapeUtils.unescapeXml(
	    						payload.getElementsByTagName("payload")
	    						.item(0)
	    						.getTextContent())));
		Document res = db.parse(inputSource);

		if (res == null) {
			out.setBody("");
			return true;
		}



		StringBuilder filter = new StringBuilder("(&(objectClass=org.eclipse.ebpm.core.ISpagicService)(SPAGIC_TYPE=SPAGIC_SERVICE)");

		// Service IDs are in <sid /> tags and connectors
		// IDs are in <cid /> :).
		NodeList sListStatus =  res.getElementsByTagName("sid");
		int max = sListStatus.getLength();
		if( max > 0) {
			filter.append("(|");

			for(int i=0; i < max; i++) {
				filter.append("(spagic.id=");
				filter.append(sListStatus.item(i).getTextContent());
				filter.append(")");
			}

			filter.append(")");
		}
		filter.append(")");

		ServiceReference services[] = 
			context
			.getBundleContext()
			.getServiceReferences(null, filter.toString());

		if (services != null) {
			max = services.length;

			for(int i=0; i < max; i++)
				serviceMap.put((String)services[i].getProperty("spagic.id"), services[i]);
		}

		filter.delete( 0, filter.length() );
		filter.append("(&(objectClass=org.eclipse.ebpm.core.ISpagicService)(SPAGIC_TYPE=SPAGIC_CONNECTOR)");

		NodeList cListStatus =  res.getElementsByTagName("cid");
		max = cListStatus.getLength();
		if( max > 0) {
			filter.append("(|");

			for(int i=0; i < max; i++) {
				filter.append("(spagic.id=");
				filter.append(cListStatus.item(i).getTextContent());
				filter.append(")");
			}

			filter.append(")");
		}
		filter.append(")");

		services = 
			context
			.getBundleContext()
			.getServiceReferences(null, filter.toString());

		if (services != null) {
			max = services.length;

			for(int i=0; i < max; i++)
				connectorMap.put((String)services[i].getProperty("spagic.id"), services[i]);
		}

		Source source = new DOMSource(createResult(db, sListStatus, cListStatus, serviceMap, connectorMap));
		StringWriter stringWriter = new StringWriter();
		Result result = new StreamResult(stringWriter);
		TransformerFactory tfactory = TransformerFactory.newInstance();
		Transformer transformer = tfactory.newTransformer();
		transformer.transform(source, result);

		Document response = db.newDocument();
		Element responseRoot = response.createElement("response");
		response.appendChild((Node)responseRoot);
		
		responseRoot.setTextContent(stringWriter.getBuffer().toString());
		
		source = new DOMSource(responseRoot);
		stringWriter = new StringWriter();
		result = new StreamResult(stringWriter);
		tfactory = TransformerFactory.newInstance();
		transformer = tfactory.newTransformer();
		transformer.transform(source, result);
		out.setBody(stringWriter.getBuffer().toString());

		return true;

	}

	private Document createResult(DocumentBuilder db,
								  NodeList sListStatus,
								  NodeList cListStatus,
								  Map<String, ServiceReference> serviceMap,
								  Map<String, ServiceReference> connectorMap) throws DOMException, Exception {

		Document resultx = db.newDocument();

		// root
		Element root = resultx.createElement("rslt");
		resultx.appendChild((Node)root);

		// Now the services
		Element svcs = resultx.createElement("svcs");
		root.appendChild(svcs);


		int max = sListStatus.getLength();
		for(int i=0; i < max; i++) {
			String id = sListStatus.item(i).getTextContent();

			Element svx =  resultx.createElement("svc");
			svx.setAttribute("id", id);

			ServiceReference s = serviceMap.get(id);

			if ( s == null )
				svx.setAttribute("status", "-1"); // not deployed
			 
			else {
				ISpagicService iss = (ISpagicService)context.getBundleContext().getService(s);
				svx.setAttribute("status", ((iss == null) ? "-1" // again, not deployed
											: ( iss.isStarted() ? "1" : "0")));
			}

			svcs.appendChild(svx);

		}

		Element cnts = resultx.createElement("cnts");
		root.appendChild(cnts);

		max = cListStatus.getLength();
		for(int i=0; i < max; i++) {
			String id = cListStatus.item(i).getTextContent();

			Element svx =  resultx.createElement("cnt");
			svx.setAttribute("id", id);

			ServiceReference s = connectorMap.get(id);

			if ( s == null )
				svx.setAttribute("status", "-1"); // not deployed
			 
			else {
				ISpagicService iss = (ISpagicService)context.getBundleContext().getService(s);
				svx.setAttribute("status", ((iss == null) ? "-1" // again, not deployed
											: ( iss.isStarted() ? "1" : "0")));
			}

			cnts.appendChild(svx);

		}

		return resultx;

	}

}
