/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.management.resources;

import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.core.ISpagicService;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.osgi.framework.ServiceReference;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

public class ServicesConnectorsToggleService extends BaseSpagicService {


	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		String s = (String)in.getBodyText();
		if (s == null) {
			out.setBody("");
			return true;
		}

		InputSource inputSource = new InputSource();
		inputSource.setCharacterStream(new StringReader(s));
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder db = factory.newDocumentBuilder();
	 
	    Document payload = db.parse(inputSource);

		String serviceId = payload.getElementsByTagName("payload")
			.item(0)
			.getTextContent();

		String newStatus = "-1";

		// We have the straight ID, we do not care if the service is a connector or a service, we
		// are going to work with ISpagicService instances anyway...
		StringBuilder filter = new StringBuilder("(&(objectClass=org.eclipse.ebpm.core.ISpagicService)(spagic.id=");
		filter.append(serviceId);
		filter.append("))");
		
		ServiceReference services[] = 
			context
			.getBundleContext()
			.getServiceReferences(null, filter.toString());

		if(services != null && services.length > 0) {
			ISpagicService iss = (ISpagicService)context.getBundleContext().getService(services[0]);

			if (iss != null) {
				if(iss.isStarted()) {
					iss.stop();
					newStatus = "0";
				}
				else {

					iss.start();
					newStatus = "1";

				}
			}
		}
		
		Document response = db.newDocument();
		Element responseRoot = response.createElement("response");
		response.appendChild((Node)responseRoot);

		responseRoot.setTextContent(newStatus);
		
		Source source = new DOMSource(responseRoot);
		StringWriter stringWriter = new StringWriter();
		Result result = new StreamResult(stringWriter);
		TransformerFactory tfactory = TransformerFactory.newInstance();
		Transformer transformer = tfactory.newTransformer();
		transformer.transform(source, result);
		out.setBody(stringWriter.getBuffer().toString());		
		return true;

	}
}
