/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.constants;

import org.eclipse.ebpm.metadb.model.TransitionState;


public class TransitionStateUtils {
	
	/**
	 * Return true if the MEP is INOUT
	 * @param transitionState Mep identifier
	 * @return true if the MEP is INOUT
	 */
	public static boolean isInOut(TransitionState transitionState) {
		int state = transitionState.getIdState();
		if ((state == TransitionStateConstants.JBI_MEP_INOUT_ACTIVE) ||
			(state == TransitionStateConstants.JBI_MEP_INOUT_OUT) ||
			(state == TransitionStateConstants.JBI_MEP_INOUT_FAULT) ||
			(state == TransitionStateConstants.JBI_MEP_INOUT_DONE))
			return true;
		return false;
	}
	
	/**
	 * Return true if the MEP is INONLY
	 * @param transitionState Mep identifier
	 * @return true if the MEP is INONLY
	 */
	public static boolean isInOnly(TransitionState transitionState) {
		int state = transitionState.getIdState();
		if ((state == TransitionStateConstants.JBI_MEP_INONLY_ACTIVE) ||
			(state == TransitionStateConstants.JBI_MEP_INONLY_DONE) ||
			(state == TransitionStateConstants.JBI_MEP_INONLY_FAULT))
			return true;
		return false;
	}

	public static boolean isFaultState(int state) {
		if (state == TransitionStateConstants.JBI_MEP_INONLY_FAULT
				|| state == TransitionStateConstants.JBI_MEP_INOPTIONALOUT_STEP1_FAULT
				|| state == TransitionStateConstants.JBI_MEP_INOPTIONALOUT_STEP2_FAULT
				|| state == TransitionStateConstants.JBI_MEP_INOUT_FAULT
				|| state == TransitionStateConstants.JBI_MEP_ROBUSTINONLY_STEP1_FAULT) {
			return true;
		} else if (state == TransitionStateConstants.BPEL_FAULTED_ACTIVITY) {
			return true;
		} else if (state == TransitionStateConstants.JBPM_FAULTED_ACTIVITY) {
			return true;
		}
		else 
			return false;
	}
	
	public static boolean isActiveState(int state) {
		if(state == TransitionStateConstants.JBI_MEP_INONLY_ACTIVE
				|| state == TransitionStateConstants.JBI_MEP_INOUT_ACTIVE
				|| state == TransitionStateConstants.JBI_MEP_INOPTIONALOUT_ACTIVE
				|| state == TransitionStateConstants.JBI_MEP_ROBUSTINONLY_ACTIVE
				|| state == TransitionStateConstants.JBI_MEP_INOUT_OUT
				|| state == TransitionStateConstants.JBI_MEP_INOPTIONALOUT_STEP1_OUT) {
			return true;
		} else if (state == TransitionStateConstants.BPEL_START_ACTIVITY) {
			return true;
		} else if (state == TransitionStateConstants.JBPM_AVAILABLE_ACTIVITY) {
			return true;
		}
		else 
			return false;
	}
	
	public static boolean isDoneState(int state) {
		if ((state == TransitionStateConstants.JBI_MEP_INONLY_DONE) ||
			(state == TransitionStateConstants.JBI_MEP_INOUT_DONE) ||
			(state == TransitionStateConstants.JBI_MEP_ROBUSTINONLY_DONE) ||
			(state == TransitionStateConstants.JBI_MEP_INOPTIONALOUT_DONE)) {
			return true;
		} else if (state == TransitionStateConstants.BPEL_END_ACTIVITY) {
			return true;
		} else if (state == TransitionStateConstants.JBPM_DONE_ACTIVITY) {
			return true;
		}
		return false;
	}
}
