/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;


/**
 * The persistent class for the attribute database table.
 * 
 */
@Entity
@Table(name="attribute")
public class Attribute implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "attribute_gen")
	@Column(name="id_attribute")
	@SequenceGenerator(name = "attribute_gen", sequenceName="seq_attribute", allocationSize = 1)
	private Long idAttribute;

	private String name;

	private int type;

	//bi-directional many-to-one association to AttributeCatalog
	@OneToMany(mappedBy="attribute")
	private List<AttributeCatalog> attributeCatalogs;

	//bi-directional many-to-many association to PoolType
	@ManyToMany(mappedBy="attributes")
	private List<PoolType> poolTypes;

	//bi-directional many-to-one association to RelevantData
	@OneToMany(mappedBy="attribute")
	private List<RelevantData> relevantData;

	//bi-directional many-to-one association to Rule
	@OneToMany(mappedBy="attribute")
	private List<Rule> rules;

    public Attribute() {
    }

	public Long getIdAttribute() {
		return this.idAttribute;
	}

	public void setIdAttribute(Long idAttribute) {
		this.idAttribute = idAttribute;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getType() {
		return this.type;
	}

	public void setType(int type) {
		this.type = type;
	}

	public List<AttributeCatalog> getAttributeCatalogs() {
		return this.attributeCatalogs;
	}

	public void setAttributeCatalogs(List<AttributeCatalog> attributeCatalogs) {
		this.attributeCatalogs = attributeCatalogs;
	}
	
	public List<PoolType> getPoolTypes() {
		return this.poolTypes;
	}

	public void setPoolTypes(List<PoolType> poolTypes) {
		this.poolTypes = poolTypes;
	}
	
	public List<RelevantData> getRelevantData() {
		return this.relevantData;
	}

	public void setRelevantData(List<RelevantData> relevantData) {
		this.relevantData = relevantData;
	}
	
	public List<Rule> getRules() {
		return this.rules;
	}

	public void setRules(List<Rule> rules) {
		this.rules = rules;
	}
	
}