/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.math.BigInteger;
import java.util.Date;


/**
 * The persistent class for the log_journal database table.
 * 
 */
@Entity
@Table(name="log_journal")
public class LogJournal implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_log_gen")
	@Column(name="id_log")
	@SequenceGenerator(name = "id_log_gen", sequenceName="seq_log_journal", allocationSize = 1)
	private Long idLog;

    @Temporal( TemporalType.TIMESTAMP)
	private Date datelog;

	@Column(name="id_log_journal_type")
	private BigInteger idLogJournalType;

    @Lob()
	private String log;

	private String name;

	//bi-directional many-to-one association to TransitionInstance
    @ManyToOne
	@JoinColumn(name="id_journal")
	private TransitionInstance transitionInstance;

	//bi-directional many-to-one association to TransitionState
    @ManyToOne
	@JoinColumn(name="id_state")
	private TransitionState transitionState;

    public LogJournal() {
    }

	public Long getIdLog() {
		return this.idLog;
	}

	public void setIdLog(Long idLog) {
		this.idLog = idLog;
	}

	public Date getDatelog() {
		return this.datelog;
	}

	public void setDatelog(Date datelog) {
		this.datelog = datelog;
	}

	public BigInteger getIdLogJournalType() {
		return this.idLogJournalType;
	}

	public void setIdLogJournalType(BigInteger idLogJournalType) {
		this.idLogJournalType = idLogJournalType;
	}

	public String getLog() {
		return this.log;
	}

	public void setLog(String log) {
		this.log = log;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public TransitionInstance getTransitionInstance() {
		return this.transitionInstance;
	}

	public void setTransitionInstance(TransitionInstance transitionInstance) {
		this.transitionInstance = transitionInstance;
	}
	
	public TransitionState getTransitionState() {
		return this.transitionState;
	}

	public void setTransitionState(TransitionState transitionState) {
		this.transitionState = transitionState;
	}
	
}