/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.Date;
import java.util.List;


/**
 * The persistent class for the process database table.
 * 
 */
@Entity
@Table(name="process")
public class Process implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_process_gen")
	@Column(name="id_process")
	@SequenceGenerator(name = "id_process_gen", sequenceName="seq_process", allocationSize = 1)
	private Long idProcess;

	private String description;

    @Temporal( TemporalType.TIMESTAMP)
	private Date enddate;

	@Column(name="monitor_enabled")
	private Boolean monitorEnabled;

	private String name;

	private String namespace;

	@Column(name="process_version")
	private String processVersion;

    @Temporal( TemporalType.TIMESTAMP)
	private Date startdate;

	private int state;

	@Column(name="uddi_name")
	private String uddiName;

	@Column(name="uddi_org")
	private String uddiOrg;

	//bi-directional many-to-one association to PoolTypeProcess
	@OneToMany(mappedBy="process")
	private List<PoolTypeProcess> poolTypeProcesses;

	//bi-directional many-to-one association to TypeService
    @ManyToOne
	@JoinColumn(name="id_ts")
	private TypeService typeService;

	//bi-directional many-to-one association to MonitoringType
    @ManyToOne
	@JoinColumn(name="id_monitoring_type")
	private MonitoringType monitoringType;

	//bi-directional one-to-one association to ProcessDiagram
	@OneToOne(mappedBy="process")
	private ProcessDiagram processDiagram;

	//bi-directional many-to-one association to ProcessInstance
	@OneToMany(mappedBy="process")
	private List<ProcessInstance> processInstances;

	//bi-directional many-to-many association to Property
	@ManyToMany(mappedBy="processes")
	private List<Property> properties;

	//bi-directional many-to-one association to Step
	@OneToMany(mappedBy="process")
	private List<Step> steps;

	//bi-directional many-to-one association to SubProcess
	@OneToMany(mappedBy="process1")
	private List<SubProcess> subProcesses1;

	//bi-directional many-to-one association to SubProcess
	@OneToMany(mappedBy="process2")
	private List<SubProcess> subProcesses2;

	//bi-directional many-to-one association to Transition
	@OneToMany(mappedBy="process")
	private List<Transition> transitions;

	//bi-directional many-to-one association to VariableDefinition
	@OneToMany(mappedBy="process")
	private List<VariableDefinition> variableDefinitions;

    public Process() {
    }

	public Long getIdProcess() {
		return this.idProcess;
	}

	public void setIdProcess(Long idProcess) {
		this.idProcess = idProcess;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Date getEnddate() {
		return this.enddate;
	}

	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}

	public Boolean getMonitorEnabled() {
		return this.monitorEnabled;
	}

	public void setMonitorEnabled(Boolean monitorEnabled) {
		this.monitorEnabled = monitorEnabled;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getNamespace() {
		return this.namespace;
	}

	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public String getProcessVersion() {
		return this.processVersion;
	}

	public void setProcessVersion(String processVersion) {
		this.processVersion = processVersion;
	}

	public Date getStartdate() {
		return this.startdate;
	}

	public void setStartdate(Date startdate) {
		this.startdate = startdate;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public String getUddiName() {
		return this.uddiName;
	}

	public void setUddiName(String uddiName) {
		this.uddiName = uddiName;
	}

	public String getUddiOrg() {
		return this.uddiOrg;
	}

	public void setUddiOrg(String uddiOrg) {
		this.uddiOrg = uddiOrg;
	}

	public List<PoolTypeProcess> getPoolTypeProcesses() {
		return this.poolTypeProcesses;
	}

	public void setPoolTypeProcesses(List<PoolTypeProcess> poolTypeProcesses) {
		this.poolTypeProcesses = poolTypeProcesses;
	}
	
	public TypeService getTypeService() {
		return this.typeService;
	}

	public void setTypeService(TypeService typeService) {
		this.typeService = typeService;
	}
	
	public MonitoringType getMonitoringType() {
		return this.monitoringType;
	}

	public void setMonitoringType(MonitoringType monitoringType) {
		this.monitoringType = monitoringType;
	}
	
	public ProcessDiagram getProcessDiagram() {
		return this.processDiagram;
	}

	public void setProcessDiagram(ProcessDiagram processDiagram) {
		this.processDiagram = processDiagram;
	}
	
	public List<ProcessInstance> getProcessInstances() {
		return this.processInstances;
	}

	public void setProcessInstances(List<ProcessInstance> processInstances) {
		this.processInstances = processInstances;
	}
	
	public List<Property> getProperties() {
		return this.properties;
	}

	public void setProperties(List<Property> properties) {
		this.properties = properties;
	}
	
	public List<Step> getSteps() {
		return this.steps;
	}

	public void setSteps(List<Step> steps) {
		this.steps = steps;
	}
	
	public List<SubProcess> getSubProcesses1() {
		return this.subProcesses1;
	}

	public void setSubProcesses1(List<SubProcess> subProcesses1) {
		this.subProcesses1 = subProcesses1;
	}
	
	public List<SubProcess> getSubProcesses2() {
		return this.subProcesses2;
	}

	public void setSubProcesses2(List<SubProcess> subProcesses2) {
		this.subProcesses2 = subProcesses2;
	}
	
	public List<Transition> getTransitions() {
		return this.transitions;
	}

	public void setTransitions(List<Transition> transitions) {
		this.transitions = transitions;
	}
	
	public List<VariableDefinition> getVariableDefinitions() {
		return this.variableDefinitions;
	}

	public void setVariableDefinitions(List<VariableDefinition> variableDefinitions) {
		this.variableDefinitions = variableDefinitions;
	}
	
}