/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;


/**
 * The persistent class for the process_instance database table.
 * 
 */
@Entity
@Table(name="process_instance")
public class ProcessInstance implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_process_instance_gen")
	@Column(name="id_process_instance")
	@SequenceGenerator(name = "id_process_instance_gen", sequenceName="seq_process_instance", allocationSize = 1)
	private Long idProcessInstance;

    @Temporal( TemporalType.TIMESTAMP)
	private Date enddate;

	private BigInteger executiontime;

	@Column(name="id_correlation")
	private String idCorrelation;

	private Boolean restarted;

	private String server;

    @Temporal( TemporalType.TIMESTAMP)
	private Date startdate;

	private int state;

	//bi-directional many-to-one association to ProcessInstance
    @ManyToOne
	@JoinColumn(name="id_parent_process_instance")
	private ProcessInstance processInstance;

	//bi-directional many-to-one association to ProcessInstance
	@OneToMany(mappedBy="processInstance")
	private List<ProcessInstance> processInstances;

	//bi-directional many-to-one association to Process
    @ManyToOne
	@JoinColumn(name="id_process")
	private Process process;

	//bi-directional many-to-one association to PoolInstance
    @ManyToOne
	@JoinColumn(name="id_pool")
	private PoolInstance poolInstance;

	//bi-directional many-to-one association to ProcessExecutionState
    @ManyToOne
	@JoinColumn(name="id_process_execution_state")
	private ProcessExecutionState processExecutionState;

	//bi-directional many-to-many association to RelevantData
	@ManyToMany(mappedBy="processInstances")
	private List<RelevantData> relevantData;

	//bi-directional many-to-one association to TransitionInstance
	@OneToMany(mappedBy="processInstance")
	private List<TransitionInstance> transitionInstances;

	//bi-directional many-to-one association to VariableInstance
	@OneToMany(mappedBy="processInstance")
	private List<VariableInstance> variableInstances;

    public ProcessInstance() {
    }

	public Long getIdProcessInstance() {
		return this.idProcessInstance;
	}

	public void setIdProcessInstance(Long idProcessInstance) {
		this.idProcessInstance = idProcessInstance;
	}

	public Date getEnddate() {
		return this.enddate;
	}

	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}

	public BigInteger getExecutiontime() {
		return this.executiontime;
	}

	public void setExecutiontime(BigInteger executiontime) {
		this.executiontime = executiontime;
	}

	public String getIdCorrelation() {
		return this.idCorrelation;
	}

	public void setIdCorrelation(String idCorrelation) {
		this.idCorrelation = idCorrelation;
	}

	public Boolean iaRestarted() {
		return this.restarted;
	}

	public void setRestarted(Boolean restarted) {
		this.restarted = restarted;
	}

	public String getServer() {
		return this.server;
	}

	public void setServer(String server) {
		this.server = server;
	}

	public Date getStartdate() {
		return this.startdate;
	}

	public void setStartdate(Date startdate) {
		this.startdate = startdate;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public ProcessInstance getProcessInstance() {
		return this.processInstance;
	}

	public void setProcessInstance(ProcessInstance processInstance) {
		this.processInstance = processInstance;
	}
	
	public List<ProcessInstance> getProcessInstances() {
		return this.processInstances;
	}

	public void setProcessInstances(List<ProcessInstance> processInstances) {
		this.processInstances = processInstances;
	}
	
	public Process getProcess() {
		return this.process;
	}

	public void setProcess(Process process) {
		this.process = process;
	}
	
	public PoolInstance getPoolInstance() {
		return this.poolInstance;
	}

	public void setPoolInstance(PoolInstance poolInstance) {
		this.poolInstance = poolInstance;
	}
	
	public ProcessExecutionState getProcessExecutionState() {
		return this.processExecutionState;
	}

	public void setProcessExecutionState(ProcessExecutionState processExecutionState) {
		this.processExecutionState = processExecutionState;
	}
	
	public List<RelevantData> getRelevantData() {
		return this.relevantData;
	}

	public void setRelevantData(List<RelevantData> relevantData) {
		this.relevantData = relevantData;
	}
	
	public List<TransitionInstance> getTransitionInstances() {
		return this.transitionInstances;
	}

	public void setTransitionInstances(List<TransitionInstance> transitionInstances) {
		this.transitionInstances = transitionInstances;
	}
	
	public List<VariableInstance> getVariableInstances() {
		return this.variableInstances;
	}

	public void setVariableInstances(List<VariableInstance> variableInstances) {
		this.variableInstances = variableInstances;
	}
	
}