/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;


/**
 * The persistent class for the property database table.
 * 
 */
@Entity
@Table(name="property")
public class Property implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_property_gen")
	@Column(name="id_property")
	@SequenceGenerator(name = "id_property_gen", sequenceName="seq_property", allocationSize = 1)
	private Long idProperty;

	private String code;

	private String value;

	//bi-directional many-to-many association to Component
    @ManyToMany
	@JoinTable(
		name="property_component"
		, joinColumns={
			@JoinColumn(name="id_property")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_component")
			}
		)
	private List<Component> components;

	//bi-directional many-to-many association to Process
    @ManyToMany
	@JoinTable(
		name="property_process"
		, joinColumns={
			@JoinColumn(name="id_property")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_process")
			}
		)
	private List<Process> processes;

	//bi-directional many-to-many association to Service
    @ManyToMany
	@JoinTable(
		name="property_service"
		, joinColumns={
			@JoinColumn(name="id_property")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_service")
			}
		)
	private List<Service> services;

	//bi-directional many-to-many association to Step
    @ManyToMany
	@JoinTable(
		name="property_step"
		, joinColumns={
			@JoinColumn(name="id_property")
			}
		, inverseJoinColumns={
			@JoinColumn(name="id_step")
			}
		)
	private List<Step> steps;

    public Property() {
    }

	public Long getIdProperty() {
		return this.idProperty;
	}

	public void setIdProperty(Long idProperty) {
		this.idProperty = idProperty;
	}

	public String getCode() {
		return this.code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getValue() {
		return this.value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public List<Component> getComponents() {
		return this.components;
	}

	public void setComponents(List<Component> components) {
		this.components = components;
	}
	
	public List<Process> getProcesses() {
		return this.processes;
	}

	public void setProcesses(List<Process> processes) {
		this.processes = processes;
	}
	
	public List<Service> getServices() {
		return this.services;
	}

	public void setServices(List<Service> services) {
		this.services = services;
	}
	
	public List<Step> getSteps() {
		return this.steps;
	}

	public void setSteps(List<Step> steps) {
		this.steps = steps;
	}
	
}