/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;

import java.util.Date;
import java.util.List;


/**
 * The persistent class for the service database table.
 * 
 */
@Entity
@Table(name="service")
public class Service implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy=GenerationType.TABLE)
	@Column(name="id_service")
	private String idService;

    @Temporal( TemporalType.TIMESTAMP)
	private Date enddate;

	@Column(name="monitor_enabled")
	private Boolean monitorEnabled;

    @Temporal( TemporalType.TIMESTAMP)
	private Date startdate;

	private int state;

	//bi-directional many-to-many association to Property
	@ManyToMany(mappedBy="services")
	private List<Property> properties;

	//bi-directional many-to-one association to Component
    @ManyToOne
	@JoinColumn(name="id_component")
	private Component component;

	//bi-directional many-to-one association to ServiceInstance
	@OneToMany(mappedBy="service")
	private List<ServiceInstance> serviceInstances;

    public Service() {
    }
    
    public Service(String idService, Component component, Date startdate) {
		this.idService = idService;
		this.component = component;
		this.startdate = startdate;
	}

	public Service(String idService, Component component, int state,
			Boolean monitorEnabled, Date startdate, Date enddate,
			List<Property> properties, List<ServiceInstance> serviceInstances) {
		this.idService = idService;
		this.component = component;
		this.state = state;
		this.monitorEnabled = monitorEnabled;
		this.startdate = startdate;
		this.enddate = enddate;
		this.properties = properties;
		this.serviceInstances = serviceInstances;
	}

	public String getIdService() {
		return this.idService;
	}

	public void setIdService(String idService) {
		this.idService = idService;
	}

	public Date getEnddate() {
		return this.enddate;
	}

	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}

	public Boolean isMonitorEnabled() {
		return this.monitorEnabled;
	}

	public void setMonitorEnabled(Boolean monitorEnabled) {
		this.monitorEnabled = monitorEnabled;
	}

	public Date getStartdate() {
		return this.startdate;
	}

	public void setStartdate(Date startdate) {
		this.startdate = startdate;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public List<Property> getProperties() {
		return this.properties;
	}

	public void setProperties(List<Property> properties) {
		this.properties = properties;
	}
	
	public Component getComponent() {
		return this.component;
	}

	public void setComponent(Component component) {
		this.component = component;
	}
	
	public List<ServiceInstance> getServiceInstances() {
		return this.serviceInstances;
	}

	public void setServiceInstances(List<ServiceInstance> serviceInstances) {
		this.serviceInstances = serviceInstances;
	}
	
}