/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.Date;
import java.util.List;


/**
 * The persistent class for the service_instance database table.
 * 
 */
@Entity
@Table(name="service_instance")
public class ServiceInstance implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_service_instance_gen")
	@Column(name="id_service_instance")
	@SequenceGenerator(name = "id_service_instance_gen", sequenceName="seq_service_instance", allocationSize = 1)
	private Long idServiceInstance;

	@Column(name="correlation_id")
	private String correlationId;

    @Temporal( TemporalType.TIMESTAMP)
	private Date enddate;

	@Column(name="message_id")
	private String messageId;

    @Lob()
	private String request;

    @Lob()
	private String response;

    @Lob()
	private String error;

    @Temporal( TemporalType.TIMESTAMP)
	private Date startdate;

	private int state;

	//bi-directional many-to-one association to PropertyServiceInstance
	@OneToMany(mappedBy="serviceInstance")
	private List<PropertyServiceInstance> propertyServiceInstances;

	//bi-directional many-to-one association to Service
    @ManyToOne
	@JoinColumn(name="id_service")
	private Service service;

	//bi-directional many-to-one association to ServiceInstance
    @ManyToOne
	@JoinColumn(name="id_target_service_instance")
	private ServiceInstance targetServiceInstance;

	//bi-directional many-to-one association to ServiceInstance
	@OneToMany(mappedBy="targetServiceInstance")
	private List<ServiceInstance> serviceInstances;

    public ServiceInstance() {
    }

	public Long getIdServiceInstance() {
		return this.idServiceInstance;
	}

	public void setIdServiceInstance(Long idServiceInstance) {
		this.idServiceInstance = idServiceInstance;
	}

	public String getCorrelationId() {
		return this.correlationId;
	}

	public void setCorrelationId(String correlationId) {
		this.correlationId = correlationId;
	}

	public Date getEnddate() {
		return this.enddate;
	}

	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}

	public String getMessageId() {
		return this.messageId;
	}

	public void setMessageId(String messageId) {
		this.messageId = messageId;
	}

	public String getRequest() {
		return this.request;
	}

	public void setRequest(String request) {
		this.request = request;
	}

	public String getResponse() {
		return this.response;
	}

	public void setResponse(String response) {
		this.response = response;
	}

	public String getError() {
		return error;
	}

	public void setError(String error) {
		this.error = error;
	}

	public Date getStartdate() {
		return this.startdate;
	}

	public void setStartdate(Date startdate) {
		this.startdate = startdate;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public List<PropertyServiceInstance> getPropertyServiceInstances() {
		return this.propertyServiceInstances;
	}

	public void setPropertyServiceInstances(List<PropertyServiceInstance> propertyServiceInstances) {
		this.propertyServiceInstances = propertyServiceInstances;
	}
	
	public Service getService() {
		return this.service;
	}

	public void setService(Service service) {
		this.service = service;
	}
	
	public ServiceInstance getTargetServiceInstance() {
		return this.targetServiceInstance;
	}

	public void setTargetServiceInstance(ServiceInstance targetServiceInstance) {
		this.targetServiceInstance = targetServiceInstance;
	}
	
	public List<ServiceInstance> getServiceInstances() {
		return this.serviceInstances;
	}

	public void setServiceInstances(List<ServiceInstance> serviceInstances) {
		this.serviceInstances = serviceInstances;
	}
	
}