/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;


/**
 * The persistent class for the step database table.
 * 
 */
@Entity
@Table(name="step")
public class Step implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_step_gen")
	@Column(name="id_step")
	@SequenceGenerator(name = "id_step_gen", sequenceName="seq_step", allocationSize = 1)
	private Long idStep;

	private String description;

    @Lob()
	private byte[] documentation;

	private String documentationFileName;

	private String documentationMimeType;

	@Column(name="has_conditional_exit")
	private Boolean hasConditionalExit;

	@Column(name="is_end")
	private Boolean isEnd;

	@Column(name="is_start")
	private Boolean isStart;

	private String name;

	private String namespace;

	@Column(name="process_version")
	private String processVersion;

	private int state;

	//bi-directional many-to-many association to Property
	@ManyToMany(mappedBy="steps")
	private List<Property> properties;

	//bi-directional many-to-one association to Process
    @ManyToOne
	@JoinColumn(name="id_process")
	private Process process;

	//bi-directional many-to-one association to StepRule
	@OneToMany(mappedBy="step")
	private List<StepRule> stepRules;

	//bi-directional many-to-one association to SubProcess
	@OneToMany(mappedBy="step")
	private List<SubProcess> subProcesses;

	//bi-directional many-to-one association to Transition
	@OneToMany(mappedBy="step1")
	private List<Transition> transitions1;

	//bi-directional many-to-one association to Transition
	@OneToMany(mappedBy="step2")
	private List<Transition> transitions2;

	//bi-directional many-to-one association to TransitionInstance
	@OneToMany(mappedBy="step1")
	private List<TransitionInstance> transitionInstances1;

	//bi-directional many-to-one association to TransitionInstance
	@OneToMany(mappedBy="step2")
	private List<TransitionInstance> transitionInstances2;

    public Step() {
    }

	public Long getIdStep() {
		return this.idStep;
	}

	public void setIdStep(Long idStep) {
		this.idStep = idStep;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public byte[] getDocumentation() {
		return this.documentation;
	}

	public void setDocumentation(byte[] documentation) {
		this.documentation = documentation;
	}

	public String getDocumentationFileName() {
		return this.documentationFileName;
	}

	public void setDocumentationFileName(String documentationFileName) {
		this.documentationFileName = documentationFileName;
	}

	public String getDocumentationMimeType() {
		return this.documentationMimeType;
	}

	public void setDocumentationMimeType(String documentationMimeType) {
		this.documentationMimeType = documentationMimeType;
	}

	public Boolean isHasConditionalExit() {
		return this.hasConditionalExit;
	}

	public void setHasConditionalExit(Boolean hasConditionalExit) {
		this.hasConditionalExit = hasConditionalExit;
	}

	public Boolean isEnd() {
		return this.isEnd;
	}

	public void setIsEnd(Boolean isEnd) {
		this.isEnd = isEnd;
	}

	public Boolean isStart() {
		return this.isStart;
	}

	public void setIsStart(Boolean isStart) {
		this.isStart = isStart;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getNamespace() {
		return this.namespace;
	}

	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public String getProcessVersion() {
		return this.processVersion;
	}

	public void setProcessVersion(String processVersion) {
		this.processVersion = processVersion;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public List<Property> getProperties() {
		return this.properties;
	}

	public void setProperties(List<Property> properties) {
		this.properties = properties;
	}
	
	public Process getProcess() {
		return this.process;
	}

	public void setProcess(Process process) {
		this.process = process;
	}
	
	public List<StepRule> getStepRules() {
		return this.stepRules;
	}

	public void setStepRules(List<StepRule> stepRules) {
		this.stepRules = stepRules;
	}
	
	public List<SubProcess> getSubProcesses() {
		return this.subProcesses;
	}

	public void setSubProcesses(List<SubProcess> subProcesses) {
		this.subProcesses = subProcesses;
	}
	
	public List<Transition> getTransitions1() {
		return this.transitions1;
	}

	public void setTransitions1(List<Transition> transitions1) {
		this.transitions1 = transitions1;
	}
	
	public List<Transition> getTransitions2() {
		return this.transitions2;
	}

	public void setTransitions2(List<Transition> transitions2) {
		this.transitions2 = transitions2;
	}
	
	public List<TransitionInstance> getTransitionInstances1() {
		return this.transitionInstances1;
	}

	public void setTransitionInstances1(List<TransitionInstance> transitionInstances1) {
		this.transitionInstances1 = transitionInstances1;
	}
	
	public List<TransitionInstance> getTransitionInstances2() {
		return this.transitionInstances2;
	}

	public void setTransitionInstances2(List<TransitionInstance> transitionInstances2) {
		this.transitionInstances2 = transitionInstances2;
	}
	
}