/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.model;

import java.io.Serializable;
import javax.persistence.*;


/**
 * The persistent class for the variable_instance database table.
 * 
 */
@Entity
@Table(name="variable_instance")
public class VariableInstance implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(generator = "id_variable_instance_gen")
	@Column(name="id_variable_instance")
	@SequenceGenerator(name = "id_variable_instance_gen", sequenceName="seq_variable_instance", allocationSize = 1)
	private Long idVariableInstance;

	private String name;

    @Lob()
	private String value;

	//bi-directional many-to-one association to ProcessInstance
    @ManyToOne
	@JoinColumn(name="id_process_instance")
	private ProcessInstance processInstance;

	//bi-directional many-to-one association to VariableDefinition
    @ManyToOne
	@JoinColumn(name="id_variable_definition")
	private VariableDefinition variableDefinition;

    public VariableInstance() {
    }

	public Long getIdVariableInstance() {
		return this.idVariableInstance;
	}

	public void setIdVariableInstance(Long idVariableInstance) {
		this.idVariableInstance = idVariableInstance;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getValue() {
		return this.value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public ProcessInstance getProcessInstance() {
		return this.processInstance;
	}

	public void setProcessInstance(ProcessInstance processInstance) {
		this.processInstance = processInstance;
	}
	
	public VariableDefinition getVariableDefinition() {
		return this.variableDefinition;
	}

	public void setVariableDefinition(VariableDefinition variableDefinition) {
		this.variableDefinition = variableDefinition;
	}
	
}