/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.metadb.utils;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import com.thoughtworks.xstream.converters.ConversionException;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

public class CalendarConverter implements Converter {

        public CalendarConverter() {
                super();
        }

        public boolean canConvert(Class clazz) {
                return Calendar.class.isAssignableFrom(clazz);
        }

        public void marshal(Object value, HierarchicalStreamWriter writer, MarshallingContext context) {
                Calendar calendar = (Calendar) value;
                Long millis = calendar.getTimeInMillis();
                String TzId = calendar.getTimeZone().getID();
                writer.setValue(millis.toString()+"#"+TzId);
        }

        public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
                GregorianCalendar calendar = new GregorianCalendar();
                try {
                		String value = reader.getValue();
                		String TzId = value.substring(value.indexOf("#")+1);
                		String strMillis = value.substring(0, value.indexOf("#"));
                		calendar.setTimeZone(TimeZone.getTimeZone(TzId));
                		calendar.setTimeInMillis(Long.parseLong(strMillis));
                } catch (Exception e) {
                        throw new ConversionException(e.getMessage(), e);
                }
                return calendar;
        }

}