/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.monitoring;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.ebpm.databasemanager.IDatabaseManager;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public class MonitorService implements EventHandler {

	private static Logger logger = LoggerFactory.getLogger(MonitorService.class);
	@SuppressWarnings("unused")
	private ComponentContext componentContext = null;
	private final AtomicReference<IDatabaseManager> dbManager = new AtomicReference<IDatabaseManager>(null); 
	private BlockingQueue<Event> eventQueue = new LinkedBlockingDeque<Event>();
	private MonitorEventConsumer eventConsumer = null;

	protected void activate(ComponentContext componentContext) {
		logger.info(" Monitor Service - ACTIVATED");
		this.componentContext = componentContext;
		eventConsumer = new MonitorEventConsumer("MonitorQueueConsumer", eventQueue, getDatabaseManager());
		eventConsumer.start();
	}

	protected void deactivate(ComponentContext componentContext) {
		logger.info(" Monitor Service - DEACTIVATED");
	}

	@Override
	public void handleEvent(Event event) {
		try{
			eventQueue.put(event);
		}catch (InterruptedException e) {
			logger.error("Interrupted", e);
		}
	}
	
	
	public void setDatabaseManager(IDatabaseManager dbManager) {
		this.dbManager.set(dbManager);
	}

	public void unsetDatabaseManager(IDatabaseManager dbManager) {
		this.dbManager.compareAndSet(dbManager, null);		
	}
	
	public IDatabaseManager getDatabaseManager() {
		return this.dbManager.get();
	}
	

}
