/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.profile.simple;


import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.eclipse.ebpm.startup.service.IStartedUpService;
import org.eclipse.ebpm.workflow.api.IProfileService;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DefaultProfileService implements IProfileService {

	private static Logger logger = LoggerFactory.getLogger(DefaultProfileService.class);
	protected List<String> dynamicRouting = new ArrayList<String>();
	protected String profilesFileName = "userprofile.xml";
	protected Timer timer = null;
	private Map<String, List<String>> userRolesMap = new HashMap<String, List<String>>();
	private File f = null;
	
	protected void loadProfiles(){
		try{
  			SAXReader reader = new SAXReader();
  			Document doc = reader.read(this.f);
  			
  			
  			Element authorizationsElement = doc.getRootElement();
  			
  			Element entitiesEl = authorizationsElement.element("ENTITIES");
  			Element usersEl = entitiesEl.element("USERS");
  			
  			Element relationsEl = authorizationsElement.element("RELATIONS");
  			Element behavioursEl = relationsEl.element("BEHAVIOURS");
  			
  			
  			List<Element> users = usersEl.elements("USER");
  			List<Element> behaviours = behavioursEl.elements("BEHAVIOUR");
  			synchronized (userRolesMap) {
  				userRolesMap = new HashMap<String, List<String>>();
	  			for (Element user: users) {
	  				String name = user.attributeValue("userID");
	  				List<String> roles = new ArrayList<String>();
	  				for (Element behaviour: behaviours) {
	  					String userid = behaviour.attributeValue("userID");//behaviour.valueOf("@userID");
	  					String role = behaviour.attributeValue("roleName");//behaviour.valueOf("@roleName");
	  					if (userid.equals(name)) {
	  						roles.add(role);
	  					}
	  				}
	  				if (roles.size() > 0)
	  					userRolesMap.put(name, roles);
	  			}
  			}

    	}catch (Exception e) {
			e.printStackTrace();
		}
      }
	
	protected void activate(ComponentContext componentContext){
		String spagicHome = System.getProperty("spagic.home");
		
		String authFile = spagicHome + File.separator + profilesFileName;
		this.f = new File(authFile);
		loadProfiles();
		TimerTask task = new FileWatcher(  this.f ) {
		      protected void onChange( File file ) {
		    	  loadProfiles();
		      }
		};

		timer = new Timer();
		timer.schedule( task , new Date(), 5000 );
		
	}
	
	protected void deactivate(ComponentContext componentContext){
		timer.cancel();
	}
	
	
	
	
	public List<String> getUserRoles(String actorId) { 
		if (userRolesMap.containsKey(actorId)){
			return userRolesMap.get(actorId);
		}else{
			return new ArrayList<String>();
		}
		 
	}

	public List<String> getUsers() {
		List<String> users = new ArrayList<String>();
		users.addAll(userRolesMap.keySet());
		return users;
	}

	public void bindStartUpService(IStartedUpService startedUpService){
		logger.info("STARTUP SERVICE IS OK -> CAN START PROFILE SERVICE");	
	}
	
	public void unbindStartUpService(IStartedUpService startedUpService){
	}
	
}

