/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.service.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Node;

public class ComboProviderFactory {
	
	private Map<String, IComboProvider> providers;

	public ComboProviderFactory() {
		init();
	}
	
	private void init() {
		providers = new HashMap<String, IComboProvider>();
	}

	public void addComboProviders(Map<? extends String, ? extends IComboProvider> m) {
		providers.putAll(m);
	}

	public IComboProvider getComboProvider(String type, List<Node> comboProviderNodes) {
		IComboProvider provider = null;
		provider = providers.get(type);
		try {
			if (provider != null) {
				IComboProvider instance = provider.getClass().newInstance();
				if (instance instanceof IConfigurableComboProvider) {
					((IConfigurableComboProvider) instance).setConfig(comboProviderNodes);
				}
				return instance;
			}
			else {
				// This should not happen unless in development, where this message could
				// save some time to the developer...
				StringBuilder sb = new StringBuilder(type);
				sb.append(" has no associated IComboProvider!!!");
				
			}
		} catch (InstantiationException e) {
			// Emergency log, but when we are here we are ***REALLY*** in trouble...
			// what are interfaces for ?
			StringBuilder sb = new StringBuilder("Very Strange! Error in the IComboPorovider instantiation: ");
			sb.append(provider.getClass().getName());
			sb.append(" seems a non instantiable class - but was instanced before... this is creepy!");
			
		} catch (IllegalAccessException e) {
			// Emergency log, but when we are here we are ***REALLY*** in trouble...
			// we CHECKED that the type was an IConfigurableComboProvider  that provides
			// setConfig()
			StringBuilder sb = new StringBuilder("Very Strange! Error accessing an IComboPorovider: ");
			sb.append(provider.getClass().getName());
			sb.append(" seems not to have the setConfig() method even if implements IConfigurableComboProvider... this is creepy!");
			
			e.printStackTrace();
		}
		return null;
	}
	
}
