/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.http.interceptors;

import java.io.InputStream;
import java.util.List;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.api.model.wsdl2.Wsdl2Message.ContentModel;
import org.eclipse.ebpm.soap.bindings.http.HttpConstants;
import org.eclipse.ebpm.soap.bindings.http.interceptors.IriDecoderHelper.Param;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpBinding;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpMessage;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpOperation;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;
import org.w3c.dom.Document;

public class HttpDecoderInterceptor extends AbstractInterceptor {

    private final boolean server;
    
    public HttpDecoderInterceptor(boolean server) {
        this.server = server;
    }
    
    public void handleMessage(Message message) {
        Operation<?> operation = message.get(Operation.class);
        if (operation instanceof Wsdl2HttpOperation == false) {
            return;
        }
        Wsdl2HttpBinding httpBinding = (Wsdl2HttpBinding) message.get(Binding.class);
        Wsdl2HttpOperation httpOperation = (Wsdl2HttpOperation) operation;
        Wsdl2HttpMessage httpMessage;
        String serialization;
        if (server) {
            httpMessage = httpOperation.getInput();
            serialization = httpOperation.getHttpInputSerialization();
        } else {
            httpMessage = httpOperation.getOutput();
            serialization = httpOperation.getHttpOutputSerialization();
        }
        if (httpMessage.getContentModel() == ContentModel.ELEMENT ||
            httpMessage.getContentModel() == ContentModel.ANY) {
            // Serialization rules for XML
            if (HttpConstants.SERIAL_APP_URLENCODED.equals(serialization)) {
                normalizeUrlEncoded(message, httpBinding, httpOperation, httpMessage);
            } else if (HttpConstants.SERIAL_MULTIPART_FORMDATA.equals(serialization)) {
                normalizeFormData(message);
            } else {
                normalizeXml(message);
            }
        } else if (httpMessage.getContentModel() == ContentModel.NONE) {
            if (message.getContent(Source.class) != null) {
                throw new Fault("No content allowed");
            }
        } else {
            throw new Fault("Unsupported content model: " + httpMessage.getContentModel());
        }
    }

    private void normalizeXml(Message message) {
        // TODO Auto-generated method stub
        
    }

    private void normalizeFormData(Message message) {
        // TODO Auto-generated method stub
        
    }

    private void normalizeUrlEncoded(Message message, Wsdl2HttpBinding binding, Wsdl2HttpOperation httpOperation, Wsdl2HttpMessage httpMessage) {
        InputStream is = message.getContent(InputStream.class);
        String uri = message.getTransportHeaders().get(HttpConstants.REQUEST_URI);
        String loc = IriDecoderHelper.combine(binding.getLocation(), httpOperation.getHttpLocation());
        List<Param> params = IriDecoderHelper.decode(uri, loc, is);
        Document doc = IriDecoderHelper.buildDocument(httpMessage.getElementDeclaration(), params);
        message.setContent(Source.class, new DOMSource(doc));
    }

}
