/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.http.interceptors;

import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.bindings.http.HttpConstants;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpBinding;
import org.eclipse.ebpm.soap.bindings.http.model.Wsdl2HttpOperation;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

public class HttpInOperationInterceptor extends AbstractInterceptor {

    public void handleMessage(Message message) {
        Binding<?> binding = message.get(Binding.class);
        if (binding instanceof Wsdl2HttpBinding == false) {
            return;
        }
        Wsdl2HttpBinding httpBinding = (Wsdl2HttpBinding) binding;
        String uri = message.getTransportHeaders().get(HttpConstants.REQUEST_URI);
        if (uri == null) {
            throw new Fault("Transport header not set: " + HttpConstants.REQUEST_URI);
        }
        String mth = message.getTransportHeaders().get(HttpConstants.REQUEST_METHOD);
        if (mth == null) {
            throw new Fault("Transport header not set: " + HttpConstants.REQUEST_METHOD);
        }
        for (Wsdl2HttpOperation operation : httpBinding.getOperations()) {
            if (mth.equalsIgnoreCase(operation.getHttpMethod())) {
                String loc = IriDecoderHelper.combine(binding.getLocation(), operation.getHttpLocation());
                String path1 = getUriPath(uri);
                String path2 = getUriPath(loc);
                if (matchPath(path1, path2)) {
                    message.put(Operation.class, operation);
                    return;
                }
            }
        }
    }

    private boolean matchPath(String path, String template) {
        int idx = template.indexOf('{');
        while (idx >= 0 && template.charAt(idx + 1) == '{') {
            idx = template.indexOf('{', idx + 2);
        }
        if (idx > 0) {
            return path.regionMatches(0, template, 0, idx - 1);
        } else {
            return path.equals(template);
        }
    }

    private String getUriPath(String uri) {
        int idx = uri.indexOf("://");
        int idx2 = uri.indexOf('/', idx + 3);
        return uri.substring(idx2 + 1);
    }

}
