/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap;

import javax.xml.namespace.QName;

/**
 * Represents the SOAP 1.2 version
 * 
 * @version $Revision: 437862 $
 */
public class Soap12 implements SoapVersion {
    
    public static final String SOAP_NAMESPACE = SoapConstants.SOAP_12_URI;
    
    public static final String SOAP_DEFAULT_PREFIX = "soap";
    
    private static final Soap12 INSTANCE = new Soap12(SOAP_DEFAULT_PREFIX);

    private final double version = 1.2;

    private final String namespace = SOAP_NAMESPACE;

    private final String prefix;

    private final String noneRole = namespace + "/role/none";

    private final String ultimateReceiverRole = namespace + "/role/ultimateReceiver";

    private final String nextRole = namespace + "/role/next";
    

    private final String soapEncodingStyle = "http://www.w3.org/2003/05/soap-encoding";

    private final QName envelope;

    private final QName header;

    private final QName body;

    private final QName fault;

    public static Soap12 getInstance() {
        return INSTANCE;
    }

    public Soap12(String prefix) {
        this.prefix = prefix;
        envelope = new QName(namespace, "Envelope", prefix);
        header = new QName(namespace, "Header", prefix);
        body = new QName(namespace, "Body", prefix);
        fault = new QName(namespace, "Fault", prefix);
    }

    public String getSoapMimeType() {
        return "application/soap+xml; charset=utf-8";
    }
    
    public double getVersion() {
        return version;
    }

    public String getNamespace() {
        return namespace;
    }

    public String getPrefix() {
        return prefix;
    }

    public QName getEnvelope() {
        return envelope;
    }

    public QName getHeader() {
        return header;
    }

    public QName getBody() {
        return body;
    }

    public QName getFault() {
        return fault;
    }
    
    public String getSoapEncodingStyle() {
        return soapEncodingStyle;
    }

    // Role URIs
    // -------------------------------------------------------------------------
    public String getNoneRole() {
        return noneRole;
    }

    public String getUltimateReceiverRole() {
        return ultimateReceiverRole;
    }

    public String getNextRole() {
        return nextRole;
    }
    
    public String getAttrNameRole() {
        return "role";
    }

    public String getAttrNameMustUnderstand() {
        return "mustUnderstand";
    }

    public SoapVersion getDerivedVersion(String prefix) {
        return new Soap12(prefix);
    }

}
