/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class SoapVersionFactory {
    private static SoapVersionFactory factory = new SoapVersionFactory();

    static {
        getInstance().register(Soap11.getInstance());
        getInstance().register(Soap12.getInstance());
    }
    
    private Map<String, SoapVersion> versions = new HashMap<String, SoapVersion>();
    
    public static SoapVersionFactory getInstance() {
        return factory;
    }
    
    public SoapVersion getSoapVersion(QName name) {
        SoapVersion v = getSoapVersion(name.getNamespaceURI());
        if (v != null && name.getPrefix() != null && !name.getPrefix().equals(v.getPrefix())) {
            v = v.getDerivedVersion(name.getPrefix());
        }
        return v;
    }
    
    public SoapVersion getSoapVersion(String namespace) {
        return versions.get(namespace);
    }
    
    public void register(SoapVersion version) {
        versions.put(version.getNamespace(), version);
    }

    public Iterator<SoapVersion> getVersions() {
        return versions.values().iterator();
    }
}
