/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap.interceptors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.bindings.soap.SoapConstants;
import org.eclipse.ebpm.soap.bindings.soap.model.SoapBinding;
import org.eclipse.ebpm.soap.bindings.soap.model.SoapOperation;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

public class SoapActionInOperationInterceptor extends AbstractInterceptor {

    public void handleMessage(Message message) {
        if (message.get(Operation.class) != null) {
            return;
        }
        String soapAction = message.getTransportHeaders().get(SoapConstants.SOAP_ACTION_HEADER);
        if (soapAction != null) {
            soapAction = soapAction.trim();
            if (soapAction.startsWith("\"") && soapAction.endsWith("\"")) {
                soapAction = soapAction.substring(1, soapAction.length() - 1);
            }
            if (soapAction.length() > 0) {
                SoapBinding<?> binding = (SoapBinding<?>) message.get(Binding.class);
                List<SoapOperation<?>> matching = new ArrayList<SoapOperation<?>>();
                for (SoapOperation<?> operation : binding.getOperations()) {
                    if (soapAction.equals(operation.getSoapAction())) {
                        matching.add(operation);
                    }
                }
                if (matching.size() == 1) {
                    message.put(Operation.class, matching.get(0));
                }
            }
        }
    }

}
