/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap.interceptors;

import java.io.OutputStream;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Namespace;
import org.dom4j.QName;
import org.dom4j.io.XMLWriter;
import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.bindings.soap.SoapVersion;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

public class SoapDOM4JOutInterceptor extends AbstractInterceptor {

    private final SoapVersion soapVersion;
    
    public SoapDOM4JOutInterceptor() {
        this(null);
    }
    
    public SoapDOM4JOutInterceptor(SoapVersion soapVersion) {
        this.soapVersion = soapVersion;
    }
    
    public void handleMessage(Message message) {
        OutputStream os = message.getContent(OutputStream.class);
       
        if (os == null) {
            throw new NullPointerException("XMLStreamWriter content not found");
        }
        SoapVersion soapVersion = message.get(SoapVersion.class); 
        if (soapVersion == null) {
            soapVersion = this.soapVersion;
            if (soapVersion == null) {
                throw new IllegalStateException("No soap version specified");
            }
        }

        try {
        	 
            XMLWriter writer = new XMLWriter(os);
        	Document doc = DocumentHelper.createDocument();
        	
        	Namespace soapNameSpace = new Namespace(soapVersion.getPrefix(), soapVersion.getNamespace());
        	
        	QName soapEnvelope = new QName(soapVersion.getEnvelope().getLocalPart(), soapNameSpace); 
        	org.dom4j.Element soapEnvelopeNode = doc.addElement(soapEnvelope);
            
        	/*
            if (message.getSoapHeaders().size() > 0) {
                for (Iterator it = message.getSoapHeaders().values().iterator(); it.hasNext();) {
                    //DocumentFragment df = (DocumentFragment) it.next();
                    //Element e = DomUtil.getFirstChildElement(df);
                }
                
            }
            */
            QName soapBody = new QName(soapVersion.getBody().getLocalPart(), soapNameSpace);
            org.dom4j.Element soapBodyNode =  soapEnvelopeNode.addElement(soapBody);
            
            
            Document nmDoc = message.getContent(Document.class);
            org.dom4j.Element bodyEl = nmDoc.getRootElement();
            bodyEl.detach();
            soapBodyNode.add(bodyEl);
            writer.write( doc );          
            writer.flush();
        } catch (Exception e) {
            throw new Fault(e);
        }
    }
    
    
}
