/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.bindings.soap.interceptors;

import java.util.Iterator;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.bindings.soap.SoapVersion;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;
import org.eclipse.ebpm.soap.util.DomUtil;
import org.eclipse.ebpm.soap.util.stax.DOMStreamReader;
import org.eclipse.ebpm.soap.util.stax.StaxUtil;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

public class SoapOutInterceptor extends AbstractInterceptor {

    private final SoapVersion soapVersion;
    
    public SoapOutInterceptor() {
        this(null);
    }
    
    public SoapOutInterceptor(SoapVersion soapVersion) {
        this.soapVersion = soapVersion;
    }
    
    public void handleMessage(Message message) {
        XMLStreamWriter writer = message.getContent(XMLStreamWriter.class);
        if (writer == null) {
            throw new NullPointerException("XMLStreamWriter content not found");
        }
        SoapVersion soapVersion = message.get(SoapVersion.class); 
        if (soapVersion == null) {
            soapVersion = this.soapVersion;
            if (soapVersion == null) {
                throw new IllegalStateException("No soap version specified");
            }
        }

        try {
            StaxUtil.writeStartElement(writer, soapVersion.getEnvelope());
            // Write Header
            if (message.getSoapHeaders().size() > 0) {
                StaxUtil.writeStartElement(writer, soapVersion.getHeader());
                for (Iterator it = message.getSoapHeaders().values().iterator(); it.hasNext();) {
                    DocumentFragment df = (DocumentFragment) it.next();
                    Element e = DomUtil.getFirstChildElement(df);
                    StaxUtil.copy(new DOMStreamReader(e), writer);
                }
                writer.writeEndElement();
            }
            // Write Body
            StaxUtil.writeStartElement(writer, soapVersion.getBody());
            // Write content
            InterceptorChain chain = message.get(InterceptorChain.class);
            chain.doIntercept(message);
            // Close elements
            writer.writeEndElement();
            writer.writeEndElement();
        } catch (XMLStreamException e) {
            throw new Fault(e);
        }
    }
    
    
}
