/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;

import org.eclipse.ebpm.soap.api.Message;
import org.w3c.dom.DocumentFragment;

/**
 */
public class MessageImpl extends HashMap<String, Object> implements Message {
    
    /**
     * Generated serial version UID 
     */
    private static final long serialVersionUID = 6062413098383260636L;
    
    private Map<String, DataHandler> attachments = new HashMap<String, DataHandler>();
    private Map<Class<?>, Object> contents = new HashMap<Class<?>, Object>();
    private Map<QName, DocumentFragment> soapHeaders = new HashMap<QName, DocumentFragment>();
    private Map<String, String> transportHeaders = new HashMap<String, String>();
    
    public Map<String, DataHandler> getAttachments() {
        return attachments;
    }

    public String getAttachmentMimeType() {
        //for sub class overriding
        return null;
    }
    
    public <T> T getContent(Class<T> format) {
        return format.cast(contents.get(format));
    }

    public <T> void setContent(Class<T> format, Object content) {
        contents.put(format, content);
    }

    public Set<Class<?>> getContentFormats() {
        return contents.keySet();
    }

    public <T> T get(Class<T> key) {
        return key.cast(get(key.getName()));
    }

    public <T> void put(Class<T> key, T value) {
        put(key.getName(), value);
    }

    public Map<QName, DocumentFragment> getSoapHeaders() {
        return soapHeaders;
    }

    public Map<String, String> getTransportHeaders() {
        return transportHeaders;
    }
}
