/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.interceptors.mime;

import java.io.ByteArrayOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.ebpm.common.util.HTTPProtocolConstants;
import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;

/**

 */
public class AttachmentsOutInterceptor extends AbstractInterceptor {

    public static final String PIPE_ATTACHMENT_STREAM = "PipeAttachmentStream";
    
    public static final String SOAP_PART_ID = "soap-request";
    
    public void handleMessage(final Message message) {
        if (message.getAttachments().size() == 0) {
            return;
        }
        OutputStream os = message.getContent(OutputStream.class);
        if (os == null) {
            throw new NullPointerException("OutputStream content not found");
        }
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            message.setContent(OutputStream.class, baos);
            message.put(Message.CONTENT_TYPE, "multipart/related; type=\"text/xml\"; start=\"<" + SOAP_PART_ID + ">\"");
            InterceptorChain chain = message.get(InterceptorChain.class);
            chain.doIntercept(message);
            writeMultipartMessage(message, os, baos.toByteArray());
        } catch (MessagingException e) {
            throw new Fault(e);
        } catch (IOException e) {
            throw new Fault(e);
        }
    }
    
    protected boolean getBoolean(Message message, String name, boolean def) {
        Object b = message.get(name);
        if (b instanceof Boolean) {
            return ((Boolean) b).booleanValue();
        } else if (b != null) {
            return Boolean.parseBoolean(b.toString());
        } else {
            return def;
        }
    }

    @SuppressWarnings("unchecked")
    private void writeMultipartMessage(Message message, OutputStream out, byte[] data) throws MessagingException, IOException {
    	 MimeMultipart parts = new MimeMultipart(HTTPProtocolConstants.HTTP_HV_RELATED);
         Session session = Session.getDefaultInstance(new Properties(), null);
         MimeMessage mime = new MimeMessage(session);
         // Add soap part
         MimeBodyPart soapPart = new MimeBodyPart();
         soapPart.setContentID("<" + SOAP_PART_ID + ">");
         soapPart.setDataHandler(new DataHandler(new ByteArrayDataSource(data, HTTPProtocolConstants.HTTP_HV_TEXT_XML)));
         soapPart.setHeader(HTTPProtocolConstants.HTTP_CONTENT_TRANSFER_ENC, HTTPProtocolConstants.HTTP_HV_ENC_BIN);
         soapPart.setHeader(HTTPProtocolConstants.HTTP_CONTENT_TYPE, HTTPProtocolConstants.HTTP_HV_TEXT_XML_UTF8);
         parts.addBodyPart(soapPart);
         // Add attachments
         for (Iterator itr = message.getAttachments().entrySet().iterator(); itr.hasNext();) {
             Map.Entry entry = (Map.Entry) itr.next();
             String id = (String) entry.getKey();
             DataHandler dh = (DataHandler) entry.getValue();
             MimeBodyPart part = new MimeBodyPart();
             part.setDataHandler(dh);
             part.setContentID("<" + id + ">"); 		
             part.setHeader(HTTPProtocolConstants.HTTP_CONTENT_TRANSFER_ENC, HTTPProtocolConstants.HTTP_HV_ENC_BIN);
             part.setHeader(HTTPProtocolConstants.HTTP_CONTENT_TYPE, HTTPProtocolConstants.HTTP_HV_APPL_OCTET);
             parts.addBodyPart(part);
         }
         mime.setContent(parts);
         mime.setHeader(HTTPProtocolConstants.HTTP_CONTENT_TYPE, parts.getContentType());
         
        
         HttpServletResponse httpResponse = message.get(HttpServletResponse.class);
//         System.out.println("1  -> "+  httpResponse.getContentType());
         httpResponse.setContentType(null);
//         System.out.println("2 -> "+  httpResponse.getContentType());
         httpResponse.setContentType(parts.getContentType());
//         System.out.println("3 -> "+  httpResponse.getContentType());
         
         // We do not want headers, so 
         //  * retrieve all headers
         //  * skip first 2 bytes (CRLF)
         mime.saveChanges();
         Enumeration<Header> headersEnum = mime.getAllHeaders();
         List<String> headersList = new ArrayList<String>();
         while (headersEnum.hasMoreElements()) {
         	String tmp = headersEnum.nextElement().getName();
             headersList.add(tmp);
         }
         String[] headers = headersList.toArray(new String[0]);
         // Skip first 2 bytes
         OutputStream os = new FilterOutputStream(out) {
             private int nb = 0;
             public void write(int b) throws IOException {
                 if (++nb > 2) {
                     super.write(b);
                 }
             }
         };
//         System.out.println("4 -> "+  httpResponse.getContentType());
         // Write
         mime.writeTo(os, headers);
    	
    	/*
    	MimeMultipart parts = new MimeMultipart("related; type=\"text/xml\"; start=\"<" + SOAP_PART_ID + ">\"");
        Session session = Session.getDefaultInstance(new Properties(), null);
        MimeMessage mime = new MimeMessage(session);
        // Add soap part
        MimeBodyPart soapPart = new MimeBodyPart();
        soapPart.setContentID("<" + SOAP_PART_ID + ">");
        soapPart.addHeader("Content-Transfer-Encoding", "8bit");
        soapPart.setDataHandler(new DataHandler(new ByteArrayDataSource(data, "text/xml")));
        parts.addBodyPart(soapPart);
        // Add attachments
        for (Iterator itr = message.getAttachments().entrySet().iterator(); itr.hasNext();) {
            Map.Entry entry = (Map.Entry) itr.next();
            String id = (String) entry.getKey();
            DataHandler dh = (DataHandler) entry.getValue();
            MimeBodyPart part = new MimeBodyPart();
            part.setDataHandler(dh);
            part.setContentID("<" + id + ">");
            part.addHeader("Content-Transfer-Encoding", "binary");
            parts.addBodyPart(part);
        }
        mime.setContent(parts);
        
        HttpServletResponse httpResponse = message.get(HttpServletResponse.class);
        httpResponse.setContentType(parts.getContentType());
        
        System.out.println("-->" +parts.getContentType());
        
        mime.setHeader(Message.CONTENT_TYPE, parts.getContentType());
        // We do not want headers, so 
        //  * retrieve all headers
        //  * skip first 2 bytes (CRLF)
        mime.saveChanges();
        

        Enumeration<Header> headersEnum = mime.getAllHeaders();
        List<String> headersList = new ArrayList<String>();
        while (headersEnum.hasMoreElements()) {
            headersList.add(headersEnum.nextElement().getName().toLowerCase());
        }
        String[] headers = headersList.toArray(new String[0]);
        // Skip first 2 bytes
        OutputStream os = new FilterOutputStream(out) {
            private int nb = 0;
            public void write(int b) throws IOException {
                if (++nb > 2) {
                    super.write(b);
                }
            }
        };
        // Write
        mime.writeTo(os, headers);
        */
    }

}
