/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.interceptors.nmr;

import java.net.URI;
import java.util.Map;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.messaging.api.Pattern;
import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.api.model.Binding;
import org.eclipse.ebpm.soap.api.model.Operation;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;
import org.eclipse.ebpm.soap.util.QNameUtil;
import org.w3c.dom.DocumentFragment;

/**

 */
public class NmrOutInterceptor extends AbstractInterceptor {

    private final boolean server;
    
    public NmrOutInterceptor(boolean server) {
        this.server = server;
    }
    
    public void handleMessage(Message message) {
        try{
        	org.eclipse.ebpm.messaging.api.Message nm = message.getContent(org.eclipse.ebpm.messaging.api.Message.class);

        Document doc = DocumentHelper.parseText(nm.getBodyText());
        message.setContent(Document.class,  doc);
        fromNMSAttachments(message, nm);
        fromNMSHeaders(message, nm);

        if (!server) {
            org.eclipse.ebpm.messaging.api.Exchange me = message.getContent(org.eclipse.ebpm.messaging.api.Exchange.class);
            Binding binding = message.get(Binding.class);
            Operation operation = binding.getOperation(me.getOperation());
            if (operation != null) {
                if (!areMepsEquals(me.getPattern(), operation.getMep())) {
                    throw new Fault("Received incorrect exchange mep.  Received " + me.getPattern()
                                    + " but expected " + operation.getMep() + " for operation "
                                    + operation.getName());
                }
                message.put(Operation.class, operation);
                message.put(org.eclipse.ebpm.soap.api.model.Message.class,
                            server ? operation.getOutput() : operation.getInput());
            }
        }
        }catch (Exception e) {
			throw new RuntimeException(e);
		}
    }

    
    private URI getURIMep(org.eclipse.ebpm.messaging.api.Pattern mepPattern){
    	if (mepPattern == org.eclipse.ebpm.messaging.api.Pattern.InOnly)
    		return SpagicConstants.IN_ONLY_URI;
    	else if (mepPattern == org.eclipse.ebpm.messaging.api.Pattern.InOut)
    		return SpagicConstants.IN_OUT_URI;
    	else
    		return null;
    }
    private boolean areMepsEquals(Pattern mepPattern, URI mep2) {
        
    	URI mep1 = getURIMep(mepPattern);
    	if (mep1 == null)
    		return false;
    	
    	String s1 = mep1 != null ? mep1.toString() : "";
        String s2 = mep2 != null ? mep2.toString() : "";
        int i1 = s1.lastIndexOf('/');
        int i2 = s2.lastIndexOf('/');
        if (i1 >= 0 && i2 >= 0) {
            return s1.substring(i1).equals(s2.substring(i2));
        }
        return false;
    }

    /**
     * Copy NormalizedMessage attachments to SoapMessage attachments
     */
    private void fromNMSAttachments(Message message, org.eclipse.ebpm.messaging.api.Message normalizedMessage) {
        
    	Map<String, Object> attachmentMap = normalizedMessage.getAttachments();
        
        for ( String attachmentId : attachmentMap.keySet()){
        	 DataHandler handler = (DataHandler)attachmentMap.get(attachmentId);
             message.getAttachments().put(attachmentId, handler);
        }
        
    }

    /**
     * Copy NormalizedMessage headers to SoapMessage headers
     */
    @SuppressWarnings("unchecked")
    private void fromNMSHeaders(Message message, org.eclipse.ebpm.messaging.api.Message normalizedMessage) {
        if (normalizedMessage.getHeader(SpagicConstants.PROTOCOL_HEADERS) != null) {
            Map<String, ?> headers = (Map<String, ?>) normalizedMessage.getHeader(SpagicConstants.PROTOCOL_HEADERS);
            for (Map.Entry<String, ?> entry : headers.entrySet()) {
                QName name = QNameUtil.parse(entry.getKey());
                if (name != null) {
                    message.getSoapHeaders().put(name, (DocumentFragment) entry.getValue());
                } else {
                    message.getTransportHeaders().put(entry.getKey(), (String) entry.getValue());
                }
            }
        }
    }

}
