/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.interceptors.xml;

import java.io.InputStream;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.eclipse.ebpm.soap.api.Fault;
import org.eclipse.ebpm.soap.api.Message;
import org.eclipse.ebpm.soap.core.AbstractInterceptor;
import org.eclipse.ebpm.soap.util.stax.DOMStreamReader;
import org.eclipse.ebpm.soap.util.stax.ExtendedXMLStreamReader;
import org.eclipse.ebpm.soap.util.stax.StaxUtil;
import org.w3c.dom.Document;

/**
 * Creates an XMLStreamReader from the InputStream on the Message.
 * 

 */
public class StaxInInterceptor extends AbstractInterceptor {

    public static final String ENCODING = "Encoding";

    public StaxInInterceptor() {
    }

    public void handleMessage(Message message) {
        try {
            XMLStreamReader reader;
            Document doc = message.getContent(Document.class);
            if (doc != null) {
                reader = new DOMStreamReader(doc.getDocumentElement());
            } else {
            	InputStream is = message.getContent(InputStream.class); 
                if (is != null) {
                    // TODO: where does encoding constant go?
                    String encoding = (String) message.get(ENCODING);
                    reader = StaxUtil.createReader(is, encoding);
                    reader = new ExtendedXMLStreamReader(reader);
                } else {
                    return;
                }
            }
            reader.nextTag();
            message.setContent(XMLStreamReader.class, reader);
        } catch (XMLStreamException e) {
            throw new Fault(e);
        }
    }

}
