/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.workflow.api;

import java.util.Map;

/**
 * 
 * @author Andrea Zoppello
 * Simplified Client API To Work With Spagic3 BPM Gateway
 */
public interface IBPMGatewayAPI {
	
	
	//
	// PROCESS AND TASK STATIC STRUCTURE DEFINITION MANAGEMENT
	//
	/**
	 * Retrieve the process definition given the processDefinitionID
	 * @param processDefinitionId
	 * @return The Process Definition Object
	 */
	public Process getProcess(long processDefinitionId);
	
	/**
	 * Retrieve the last process definition given the process Name
	 * @return The Process Definition Object
	 */
	public Process getProcess(String processName);
	public Process getChildrenProcess(long processDefinitionId);
	public Process getChildrenProcess(String processName);
	public Process[] getProcesses(boolean onlyStandalone);
	public Process[] getProcesses(boolean onlyStandalone, Map<String,String> filterOnProcessDefinitionStaticAttributes);
	
	/**
	 * Retrieve the task definitions for process identified by processDefinitionId
	 * @param processDefinitionId
	 * @return The array of Task definition object
	 */
	public Task[] getTaskDefinitions(long processDefinitionId);
	
	
	/**
	 * Retrieve the task defintions for process identified by processDefinitionId
	 * @param processDefinitionId
	 * @return The array of Task definition object
	 */
	public Task[] getTaskDefinitions(long processDefinitionId, Map<String,String> filterOnTaskStaticAttributes);
	
	/**
	 * Gets all the task of the process that are subprocess
	 * 
	 * @param processId - The process identifier
	 * @return - The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTaskDefinitions(long processDefinitionId);
	
	//
	// PROCESS  MANAGEMENT
	//
	/**
	 * Start a new process instance for the process identified by the processDefinitionID
	 * @param processDefinitionId
	 * @return The ProcessInstance
	 */
	public ProcessInstance startProcess(long processDefinitionId);
	
	/**
	 * Start a new process instance for the process identified by the processDefinitionID passing
	 * an initial set of variable
	 * @param processDefinitionId
	 * @param initialVariables 
	 * @return The ProcessInstance 
	 */
	public ProcessInstance startProcess(long processDefinitionId, Variable[] initialVariables);
	public ProcessInstance[] getLinkedProcessInstances(long processInstanceId);
	public ProcessInstance[] getChildrenProcessInstances(long processInstanceId);
	public void linkProcessInstances(long firstProcessInstanceId, long secondInstanceId);
	
	//
	// TASK MANAGEMENT
	//
	public TaskInstance getTasks(long processInstanceId);
	public TaskInstance getTasks(long processInstanceId, String actor);
	public TaskInstance getTasks(long processInstanceId, String[] actors);
	public void acceptTask(long taskInstanceId, Variable[] variables);
	public void completeTask(long taskInstanceId, Variable[] variables);
	public void advanceTaskCompletion(long taskInstanceId, int percentage, Variable[] variables);
	public void acceptTask(long taskInstanceId, String actor,Variable[] variables);
	public void completeTask(long taskInstanceId, String actor, Variable[] variables);
	public void advanceTaskCompletion(long taskInstanceId, int percentage, String actor, Variable[] variables);
	public void rejectTask(long taskInstanceId);

	//
	// VARIABLE MANAGEMENT
	//
	public Variable getVariable(long processInstanceId, String variableName);
	public Variable[] getVariables(long processInstanceId);
	public void setVariables(long processInstanceId, Variable[] vars);
	public void setVariables(long processInstanceId, Variable v);
	
	
	//
	// TASK VARIABLES
	//
	public Variable[] getTaskVariables(long taskInstanceId);
	public Variable getTaskVariable(long taskInstanceId, String variableName);
	public void setTaskVariables(long taskInstanceId, Variable[] vars);
	
	public void setTaskVariable(long taskInstanceId, Variable v);
	
	
}
