/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm;

import org.eclipse.ebpm.workflow.api.IControlAPI;
import org.eclipse.ebpm.workflow.api.IControlExtendedAPI;
import org.eclipse.ebpm.workflow.api.IProcessEngine;
import org.eclipse.ebpm.workflow.api.IProfileService;
import org.eclipse.ebpm.workflow.api.IQueryAPI;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.workflow.api.jbpm.ProcessEngine;

/**
 * 
 * @author zoppello
 *
 * This class wrap the JBPM Process Engine, making it available only when
 * the datasource with name JBPM is available
 */
public class OSGiBPMProcessEngine implements IProcessEngine {
	private Logger logger = LoggerFactory.getLogger(OSGiBPMProcessEngine.class);

	private IProcessEngine delegatetProcessEngine = null;
	private IProfileService profileService = null;
	
	protected void activate(ComponentContext componentContext){
		delegatetProcessEngine = new ProcessEngine();
		delegatetProcessEngine.setProfileService(this.profileService);
	}
	
	protected void deactivate(ComponentContext componentContext){
		stop();
		delegatetProcessEngine.setProfileService(null);
		delegatetProcessEngine = null;
	}

	@Override
	public IControlAPI getControlAPI() {
		return delegatetProcessEngine.getControlAPI();
	}

	@Override
	public IControlExtendedAPI getControlExtendedAPI() {
		return delegatetProcessEngine.getControlExtendedAPI();
	}

	@Override
	public IQueryAPI getQueryAPI() {
		
		return delegatetProcessEngine.getQueryAPI();
	}

	@Override
	public void stop() {
		delegatetProcessEngine.stop();
	}
	
	public void bindJBPM(javax.sql.DataSource ds){
		logger.info("JBPM Datasource has been bound");
	}
	
	public void unbindJBPM(javax.sql.DataSource ds){
		logger.info("JBPM Datasource has been unbound");
	}
	
	public void bindMetaDB(javax.sql.DataSource ds){
		logger.info("Metadb Datasource has been bound");
	}
	
	public void unbindMetaDB(javax.sql.DataSource ds){
		logger.info("Metadb Datasource has been unbound");
	}

	
	public void setProfileService(IProfileService profileService) {
		
		this.profileService = profileService;
	}
	
	public void unsetProfileService(IProfileService profileService) {
		this.profileService = null;
	}

	
	public IProfileService getProfileService() {
		return this.profileService;
	}


}
