/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.handler;

import java.util.Map;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.core.ProcessContext;
import org.eclipse.ebpm.integration.api.IExchangeProvider;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Pattern;
import org.eclipse.ebpm.messaging.core.MessageImpl;
import org.jbpm.context.exe.ContextInstance;
import org.jbpm.graph.exe.ExecutionContext;
import org.jbpm.graph.exe.Token;
import org.spagic.workflow.api.jbpm.TokenContext;
import org.spagic3.components.bpm.BPMContextSingleton;

public class VariableIntoMessageExchangeProvider implements IExchangeProvider {

	
	public Exchange createExchangeFromWorkflowContext(Object workflowContext) {
		ExecutionContext context = (ExecutionContext)workflowContext;
		ContextInstance contextInstance = context.getContextInstance();
		
		Exchange exchange = ExchangeUtils.createExchange(Pattern.InOut);
		String xmlMessage = (String)contextInstance.getVariable(BPMContextSingleton.XML_MESSAGE, context.getToken());
		Message in = exchange.getIn(true);
		in.setBody(xmlMessage);
		
		// Fill the Exchange with Attachment List
		String attachments = (String)context.getContextInstance().getVariable(BPMContextSingleton.ATTACHMENTS, context.getToken());
		if (attachments != null){
			((MessageImpl)in)._decodeAttachments(attachments);
		}
		
		// Fill the Exchange with header List
		String msgHeaders = (String)context.getContextInstance().getVariable(BPMContextSingleton.MESSAGE_HEADERS, context.getToken());
		if (msgHeaders != null){
			ExchangeUtils.fillHeaders(in, msgHeaders);
		}
		TokenContext tokenContex = TokenContext.fromExecutionContext(context);
		exchange.setProperty(BPMContextSingleton.TOKEN_CONTEXT, tokenContex.toString());

		setSyncExecutionIfNeeded(context, exchange);
		
		setVariablesIntoExchange(contextInstance, context.getToken(), exchange);
		// Provide Spagic process instance id as an exchange property
		setProcessInstanceIdIntoExchange(contextInstance, context.getToken(), exchange);

		return exchange;
	}

	protected void setSyncExecutionIfNeeded(ExecutionContext workflowContext, Exchange exchange){
		String requireSyncExecutionString = (String)workflowContext.getVariable(BPMContextSingleton.REQUIRE_BPM_SYNC_EXECUTION);
		if ((requireSyncExecutionString != null) && (requireSyncExecutionString.equalsIgnoreCase("TRUE"))){
			exchange.setProperty(SpagicConstants.SYNC_EXCHANGE, "true");
		}
	}
	
	public Map<String, String> getProperties() {
		// TODO Auto-generated method stub
		return null;
	}

	
	public void setProperties(Map<String, String> properties) {
		// TODO Auto-generated method stub
		
	}

	public void setVariablesIntoExchange(ContextInstance contextInstance, Token token, Exchange exchange){
		ProcessContext pc = new ProcessContext(exchange);
		Map<String, Object> varMaps = contextInstance.getVariables(token);
		for (String var : varMaps.keySet()){
			if ( var.equalsIgnoreCase(BPMContextSingleton.XML_MESSAGE)  
			   || var.equalsIgnoreCase(BPMContextSingleton.ATTACHMENTS)
			   || var.equalsIgnoreCase(BPMContextSingleton.TOKEN_CONTEXT)
			   || var.equalsIgnoreCase(BPMContextSingleton.REQUIRE_BPM_SYNC_EXECUTION)
			   || var.equalsIgnoreCase(BPMContextSingleton.ORCHESTRATION_SERVICE_ID)
			   || var.equalsIgnoreCase(BPMContextSingleton.MESSAGE_HEADERS)
			   || var.equalsIgnoreCase(SpagicConstants.WF_PROCESS_INSTANCE_ID_VARIABLE)){
				continue;
			}else{
				pc.setVariable(var, varMaps.get(var));
			}
		}
	}
	
	public void setProcessInstanceIdIntoExchange(ContextInstance contextInstance, Token token, Exchange exchange) {
		Object var = contextInstance.getVariable(SpagicConstants.WF_PROCESS_INSTANCE_ID_VARIABLE, token);
		if (var != null) {
			exchange.setProperty(SpagicConstants.WF_PROCESS_INSTANCE_ID_VARIABLE, var);
		}
	}
	

}
