package org.spagic3.components.drools;


import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.core.ProcessContext;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.util.resources.IResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.components.drools.helper.MessageHelper;


public class DroolsComponent extends BaseSpagicService {
		

	protected Logger logger = LoggerFactory.getLogger(DroolsComponent.class);
	
	private IResource rulesFile = null;
	private KnowledgeBase kbase;
	
	public void init() {
		try {
			this.rulesFile = propertyConfigurator.getResource("rulesFile");
			logger.debug("rulesFile = " + rulesFile);
			this.kbase = readKnowledgeBase(rulesFile);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate Drools Component", e);
		}
	}
	
	private KnowledgeBase readKnowledgeBase(IResource rulesFile) throws Exception {
		logger.debug("reading knowledge base");
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newInputStreamResource(rulesFile.openStream()), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error : errors) {
				logger.error(error.getMessage());
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}
	
	public void beforeDestroy() {}
    
	public boolean run(Exchange exchange, Message in, Message out) throws Exception {
		try {
			logger.debug("processing exchange " + exchange.getId());
			logger.debug("original sender = " + 
					(String) exchange.getProperty(SpagicConstants.SPAGIC_SENDER));
			logger.debug("original target = " + 
					(String) exchange.getProperty(SpagicConstants.SPAGIC_TARGET));
		   
			logger.debug("opening stateful knowledge base session");
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
	
			logger.debug("inserting exchange");
			ksession.insert(exchange);
	
			logger.debug("inserting message helper");
			MessageHelper messageHelper = new MessageHelper(exchange);
			ksession.insert(messageHelper);
	
			logger.debug("inserting process context");
			ProcessContext processContext = new ProcessContext(exchange);
			ksession.insert(processContext);
			
			// start session
			logger.debug("starting session");
			ksession.fireAllRules();
			
			return true;
		} catch (Exception e) {
	 		logger.error(" Component Error", e);
	     	throw new Exception("Failed to fire rules. Reason: " + e, e);
	 	}
	}
	
}
