/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.exchanges;

import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.osgi.framework.ServiceReference;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExchangeStoreManager implements IExchangeStoreManager {

	protected ConcurrentHashMap<String, ServiceReference> exchangeStore = new ConcurrentHashMap<String, ServiceReference>();
	private static Logger logger = LoggerFactory.getLogger(ExchangeStoreManager.class);
	private ComponentContext componentContext = null;
	
	protected void activate(ComponentContext componentContext) {
		logger.info(" ExchangeStoreManager - ACTIVATED");
		this.componentContext = componentContext;
	}

	protected void deactivate(ComponentContext componentContext) {
		logger.info(" ExchangeStoreManager - DEACTIVATED");
		
	}
	public Exchange deleteExchange(Long pid) {	
		return getExchangeStore().delete(pid);
	}

	
	public Exchange loadExchange(Long pid) {
		return getExchangeStore().load(pid);
	}

	
	public void storeExchange(Long pid, Exchange ex) {
		getExchangeStore().store(pid, ex);
	}

	protected IExchangeStore getExchangeStore(){
		String exchangeStoreSetted = System.getProperty(SpagicConstants.EXCHANGE_STORE);
		if (exchangeStoreSetted == null)
			exchangeStoreSetted = SpagicConstants.EXCHANGE_STORE_OSGI_PREF;
		
		ServiceReference sr = exchangeStore.get(exchangeStoreSetted);
		IExchangeStore selectedStore = (IExchangeStore)componentContext.locateService("exstores", sr);
		if (selectedStore == null)
			throw new RuntimeException(" Error in getting Exchange Store");
		
		return selectedStore;
	}
	
	
	public void addExStore(ServiceReference exstoreServiceReference) {
		
		String identifier = (String) exstoreServiceReference.getProperty("id");
		
		logger.info(" ExchangeStore ["+identifier+"] -- REGISTERED");
		
		
		exchangeStore.put(identifier, exstoreServiceReference);
		
	}

	public void removeDataSource(
			ServiceReference exstoreServiceReference) {

		String identifier = (String) exstoreServiceReference.getProperty("id");
		
		logger.info(" ExchangeStore ["+identifier+"] -- UNREGISTERED");
		exchangeStore.remove(identifier);
		

	}

	
}
