package org.spagic3.jmsconnectionfactory.jboss;

import java.util.Properties;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConnectionFactory implements org.eclipse.ebpm.jms.connectionfactory.ConnectionFactory {
	
	private Logger logger = LoggerFactory.getLogger(ConnectionFactory.class);
	private javax.jms.ConnectionFactory cf = null;
	private InitialContext initialCtx = null;
	
	@Override
	public javax.jms.Connection createConnection() throws JMSException {
		logger.debug("Creating connection");
		return cf.createConnection();
	}

	@Override
	public javax.jms.Connection createConnection(String userName, String password)
			throws JMSException {
		logger.debug("Creating connection");
		return cf.createConnection(userName, password);
	}

	protected void activate(ComponentContext componentContext){
		try{
			PropertyConfigurator propertyConfigurator = new PropertyConfigurator(componentContext.getProperties());
			String providerUrl = propertyConfigurator.getString("java.naming.provider.url", "jnp://localhost:1099");
			String factoryUrlPkgs = propertyConfigurator.getString("java.naming.factory.url.pkgs", "org.jboss.naming");
			String factoryInitial = propertyConfigurator.getString("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
			String jndiCFName = propertyConfigurator.getString("jndiConnectionFactoryName", "/ConnectionFactory");
			
			Properties props = new Properties();
			props.setProperty("java.naming.provider.url", providerUrl);
			props.setProperty("java.naming.factory.url.pkgs", factoryUrlPkgs);
			props.setProperty("java.naming.factory.initial", factoryInitial);

			initialCtx = new InitialContext(props);
		    cf = (javax.jms.ConnectionFactory)initialCtx.lookup(jndiCFName);
		}catch (Exception e) {
			throw new IllegalStateException("Cannot istantiate ConnectionFactory", e);
		}
	}

	@Override
	public Destination getDestination(boolean pubSubDomain, String destinationName) throws NamingException {
		Destination result = (Destination)initialCtx.lookup((pubSubDomain ? "/topic/" : "/queue/") + destinationName);
		return result;
	}
		
}
