/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.core.servicedefinitions.extensionprovider;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.eclipse.ebpm.service.model.IComboProvider;
import org.eclipse.ebpm.service.model.ServiceModelHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public abstract class AbstractServiceDefinition implements IServiceDefinition {

	public static final String CONNECTORS = "/scrappy/connectors/connector";
	public static final String SERVICES = "/scrappy/services/service";
	public static final String DEFINITIONS = "/scrappy/definitions/def";
	public static final String COMBOPROVIDERS = "/scrappy/combo-providers/combo-provider";
	public static final String DEPLOYSCHEMAS = "/scrappy/deploy-schemas/deploy-schema";

	private Document serviceDefDocument;
	private Map<String, IComboProvider> comboProviderFactoryContribution; 

	public Document getServiceDefDocument() {
		return serviceDefDocument;
	}

	public void setServiceDefDocument(Document serviceDefDocument) {
		this.serviceDefDocument = serviceDefDocument;
	}

	public Map<String, IComboProvider> getComboProviderFactoryContribution() {
		return comboProviderFactoryContribution;
	}

	public void setComboProviderFactoryContribution(
			Map<String, IComboProvider> comboProviderFactoryContribution) {
		this.comboProviderFactoryContribution = comboProviderFactoryContribution;
	}

	public List<Node> getNodeList(String path) {
		if (serviceDefDocument == null) {
			return null;
		}
		XPath xPath = XPathFactory.newInstance().newXPath();
		xPath.setNamespaceContext(ServiceModelHelper.getNsContext());
		NodeList nodes = null;
		try {
			nodes = (NodeList) xPath.evaluate(path, serviceDefDocument,
					XPathConstants.NODESET);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		List<Node> result = new ArrayList<Node>(nodes.getLength());
		for (int i = 0; i < nodes.getLength(); i++) {
			result.add(nodes.item(i));
		}
		return result;
	}

	public List<Node> getDeploySchemas() {
		return getNodeList(DEPLOYSCHEMAS);
	}

	public List<Node> getComboProviders() {
		return getNodeList(COMBOPROVIDERS);
	}

	public List<Node> getConnectors() {
		return getNodeList(CONNECTORS);
	}

	public List<Node> getDefinitions() {
		return getNodeList(DEFINITIONS);
	}

	public List<Node> getServices() {
		return getNodeList(SERVICES);
	}
	

}
